/* Stats - IRC Statistical Services for Hybrid, csircd, and IRCnet 2.10
** config.h - Hard coded configuration information
**
** Copyright W. Campbell and others.  See README for more details
** Some code Copyright: Jonathan George, Kai Seidler, ircd-hybrid Team,
**                      IRCnet IRCD developers.
**
** $Id: config.h,v 1.56 2002/10/29 20:00:46 wcampbel Exp $
*/

#ifndef CONFIG_H
#define CONFIG_H

/* These path options MUST be set */

/* This is the root directory of the Stats distribution, where the etc/
** bin/ and logs/ directories are
**
** WARNING:  If this is not set, Stats WILL NOT start up!
*/

#define DPATH "/home/user/Stats/"

/* You MUST set this or RESTART won't work!!!!
** Set this to the path to the stats binary
*/

#define SPATH "/home/user/Stats/bin/stats"

/* Define this to enable encrypted passwords in the user file */
#define CRYPT

/* Define this to use MD5 as opposed to DES passwords (requires CRYPT
** to be defined).  This cannot be used on Solaris, Mac OS-X, or current
** Cygwin systems.
*/
#undef MD5

/* Define this to hide all cases where the uplink's hostname/IP can show
** ...even to opers.
**
** The only way to see the uplink's hostname is by looking in the conf
** file.
*/
#undef HIDE_IPS

/* Enable the more secure method of communicating with the service, 
** PRIVMSG user@server :message.  This is quite stable and usable, and is
** now enabled by default.  It does SOMEWHAT increase CPU utilization, but
** not by much
*/

#define SUPPORT_MSG_UH

/* Only allow the secure method of communicating with the service, disallowing
** PRIVMSG nick :message.
*/

#undef DISALLOW_NICK_PRIVMSG

/* This is another new feature... This is for support of the Hybrid 6
** NOTICE/PRIVMSG opers@server feature.  If this is enabled, all messages
** to opers@<my name> will be forwarded on to all logged in users.
** This does increase CPU usage so it is disabled by default
*/

#undef SUPPORT_MSG_OPERS

/* Define this to have the service ignore illegal top level domains
** This is useful on servers that allow spoofs for either opers 
** or users.  Currently, this will just prevent logging of the illegal
** TLD's to the log file.  If there is a valid ICANN approved TLD that is
** not working, please let me know so I can fix it.  
**
** NOTE:  I am not putting the Open Root Servers or the ADNS TLD's into
**        the official distribution.  If you use one or the other and
**        wish to have support, then you will have to create a TLD
**        table.  I could then distribute this TLD table along side the
**        official one.
**
** NOTE:  Your statistics are not changed by this option.  Any unknown
**        tld users are assigned to the 'unknown' TLD, and properly
**        shown in TLDMAP
*/

#undef IGNORE_ILLEGAL_TLD

/* If you wish the Stats service to track server VERSION's, and report them
** on a SERVER request and on the HTML dump, define this.  It will increase
** memory usage, CPU, and bandwidth on the network.  It won't be terribly
** noticeable though, and may be worth the extra
*/

#undef TRACK_VERSION

/* If you wish the Stats service to track server ADMIN replies, and report
** them on a SERVER request and on the HTML dump, define this.  It will
** increase the memory usage, CPU, and bandwidth on the network, and may
** annoy opers.  It won't be terribly noticable though, and may be worth
** anything it does.
*/

#undef TRACK_ADMIN

/* On Hybrid 7 (and Hybrid 6 if you have an admin mode patch), this option
** will show opers as A O o (with o most likely never being used)
**
** Idea from Xride's ModeServ for Dal
*/

#define OPER_MODES

/* This will enable or disable QUOTE/RAW
** #define it to have it enabled
**
** WARNING:  Use this at your own risk!
*/

#undef ENABLE_QUOTE

/* Enable this option to create a 3/4 second artificial delay between
** sequential PRIVMSG's.  Note that this can and will lag your service.
** This option is ONLY in the conf to support servers with extremely
** strict flood control, enough to rate limit remote users... (Hyb7)
**
** NOTE:  This option shouldn't be necessary after Hybrid-7 beta2
*/

#undef DELAY_PRIVMSG

/* Large network support - The targeted audience of this package
** are small networks (Under 1000 users and 60 servers)
** Enabling this option will enlarge the data tables (and increase
** memory usage).  This is untested, and therefore may cause problems
** I would avoid enabling this until there is further testing
**
** This service will need to be heavily optimized to support a large
** number of users.  The optimization HAS begun, but is far from complete.
*/

#undef LARGE_NET

/* Enable the option that will cause the service to gracefully exit
** after a certain number of collisions in a certain period of time
** This prevents 2 services running with the same nickname from massively
** colliding each other.  Disable this on a centrally controlled network, 
** where issues like this cannot occur.
**
** NOTE:  This does mean that an abusive admin can force the service to 
**        die quite easily.  Tradeoffs.....
*/

#define DIE_ON_MASS_COLLIDE

/* The spam service has been removed.  There are far better ways to track
** spam, use one of them.
*/

/* Define this option to enable the Split service.  This is the code that
** was part of my old Split service, now incorporated into the superior
** Stats code.
** This code is now appearing to work with the same functionality as
** the old Split code was.
*/

#undef SPLITSERV

/* Option to SplitServ, if this is defined, any server that was on your
** network prior to this service becoming disconnected from it's uplink
** will be marked as "Split".  When the service returns to the network,
** it will report those servers being introduced back as "End of Split"
** This is used primarily for test networks, with lots of servers
*/

#undef TRACK_SPLITS_ON_DISCONNECT

/* Define this option to enable the Statistical service.  This is here to
** disable by people who may want to run a split service only, or have
** a service that has no user interface, only a web dump.  This will ONLY
** prevent the bot from appearing on the network.  Statistics are still
** properly kept.
** This is enabled by default
*/

#define STATSERV

/* Define this option to enable the Jupe service.  Jupe is a touchy feature,
** so it is off by default.
**
** NOTE:  JupeServ will not currently work on IRCnet 2.10.x mode.
*/

#undef JUPESERV

/* Define this if you use HostServ or another program that changes hostnames
** or usernames of users.  This implementation handles the pseudostandard
** commands SVSHOST, CHGHOST, and SETHOST for host changing, and SVSIDENT,
** CHGIDENT, and SETIDENT for username changing.
*/
#undef SUPPORT_HOSTSERV

/* The Administraion Service can now report things to channels.
** There are currently two different options here, both of which chan
** be independently enabled.
*/

/* ALERTMODE will send important messages that are logged to ALERTCHAN.
** Don't count on the channel being a define though, it may soon go into
** the conf.
*/

#undef ALERTMODE
#define ALERTCHAN "#stats-alert"

/* SPYMODE will send client connect/disconnects, non command user PRIVMSG's,
** and misc commands to the service to SPYCHAN
**
** This is an invasion of privacy, and I do not support it.
*/

#undef SPYMODE
#define SPYCHAN "#stats-spy"

/* DCC options, these are the ports it will use to issue DCC CHAT requests
** from.  These ports have to be able to be reached through any firewall
** in the way.
*/
#define DCC_LOW_PORT	3510
#define DCC_HIGH_PORT	3590

/* Only enable ONE - There is no checking or sanity checking on this 
** option yet...
**
** Supported types...
** IRCD_HYBRID
** IRCD_HYB6
** IRCD_HYB7
** IRCD_2_10
** IRCD_BAHAMUT
** IRCD_HYBSVS
**
** Look at the README file for more detailed descriptions of these IRCD
** types.
*/
/* #define IRCD_HYBRID */
#define IRCD_HYB6
/* #define IRCD_HYB7 */
/* #define IRCD_2_10 */
/* #define IRCD_BAHAMUT */
/* #define IRCD_HYBSVS */

/* STOP STOP */
#ifdef IRCD_2_10
#undef OPER_MODES
#undef JUPESERV
#endif

#endif /* CONFIG_H */
