/* Stats - IRC Statistical Services for Hybrid, csircd, and IRCnet 2.10
** struct.h - Structure definitions
**
** Copyright W. Campbell and others.  See README for more details
** Some code Copyright: Jonathan George, Kai Seidler, ircd-hybrid Team,
**                      IRCnet IRCD developers.
**
** $Id: struct.h,v 1.90 2002/10/29 19:30:28 wcampbel Exp $
*/

#ifndef STRUCT_H
#define STRUCT_H

#include "stats.h"

/* The old bit fields are a gcc feature...replace it with bit flags
** and check with & and |
*/
#define FLAG_NOTIFYEOB	0x00000001
#define FLAG_PRIVMSG	0x00000002
#define FLAG_ONLYOPERS	0x00000008
#define FLAG_HTML	0x00000010
#define FLAG_SETOPER	0x00000020
#define FLAG_DUMP	0x00000040
#define FLAG_HIDE	0x00000080
#define FLAG_FLOOD	0x00000100
#define FLAG_VHOST	0x00000200
#define FLAG_AONLYOPERS 0x00000400

/* For JupeServ */
#define FLAG_JUPE_PERSIST	0x00000400

#define FLAG_SILENT	0x00000800
#define FLAG_UNAUTHCHAT	0x00001000
#define FLAG_DCCOPERONLY	0x00002000

#define UFLAG_FLOODSENT	0x00000001
#define UFLAG_ISOPER	0x00000002
#define UFLAG_ISAWAY	0x00000004

#define MFLAG_ISON	0x00000001
#define MFLAG_DCC	0x00000002

#define SFLAG_HIDE	0x00000001
#define SFLAG_EOB	0x00000002

#define CMD_DEBUG       0x00000001   
#define CMD_NOFORK      0x00000002   

#define OPER_LOCAL	0x00000001
#define OPER_GLOBAL	0x00000002
#define OPER_ADMIN	0x00000004
#define OPER_MYOPER	0x00000008
#define OPER_SVSADMIN	0x00000010

#define SPF_PRIVMSG	0x00000001
#define SPF_CHANMSG	0x00000002
#define SPF_ENABLE_TEXT	0x00000004
#define SPF_REPORT	0x00000008
#define SPF_EXPIRE	0x00000010
#define SPF_HIDE	0x00000020

/* Since there are so many ways that messages can be sent,
** privmsg() now requires a flag to specify what type of
** message to send.  Currently supported types are:
** MSG_DEFAULT - The default according to WANT_PRIVMSG in
**               the conf
** MSG_SPDEFAULT - The default according to SPLITSERV_PRIVMSG
**                 or SPLITSERV_CHANMSG
** MSG_PRIVMSG - Force PRIVMSG to be used
** MSG_NOTICE - Force NOTICE to be used
** MSG_SQUERY - Not currently used, but can be used in extensions
**              to send an SQUERY message to another service
*/

#define MSG_DEFAULT	0
#define MSG_SPDEFAULT	1
#define MSG_PRIVMSG	2
#define MSG_NOTICE	3
#define MSG_SQUERY	4

/* SplitServ has some options to define which functions are available to
** whom.  They are defined here.
*/

/* Default to OPEN */
#define SO_OPEN		0
#define SO_OPERONLY	1
#define SO_DISABLED	2

#define CONN_SERV	0x00000001
#define CONN_DCCOUT	0x00000002
#define CONN_DCCIN	0x00000004
#define CONN_DEAD	0x00000008
#define CONN_CONNECTING	0x00000010
#define CONN_NOECHO	0x00000020

/* To save some space, I will include the "usermodes" into the same flags
** variable
*/
#define CF_AUTH		0x80000000
#define CF_NOTICES	0x40000000
#define CF_JUPE		0x20000000
#define CF_OPER		0x10000000
#define CF_COMMANDS	0x08000000
#define CF_USER		0x04000000
#define CF_KILL		0x02000000
#define CF_MISC		0x01000000
#define CF_SERVICES	0x00800000
#define CF_TLD		0x00400000
#define CF_CHAT		0x00200000

/* And an aggregate for .flags all and .flags -all */
#define CF_ALL		(CF_AUTH|CF_NOTICES|CF_JUPE|CF_OPER|CF_COMMANDS|\
			 CF_USER|CF_KILL|CF_MISC|CF_SERVICES|CF_TLD|CF_CHAT)

struct netinfo
{
	long inb;
	long outb;
        int opers;     
        int maxopers;
        long users;
        long maxusers;
        int servers;
        int maxservers;
        time_t t_maxopers;  
        time_t t_maxusers;  
        time_t t_maxservers;
        int away; 
	int maxaway;
	time_t t_maxaway;
        int requests;
#ifdef IRCD_2_10
	int services;
	int maxservices;
	time_t t_maxservices;
#endif
#ifdef DIE_ON_MASS_COLLIDE
        int collides;
#endif
} netinfo;

struct settings
{
        char name[MAXHOST];
        int port;
        unsigned int r_time;
        char uplink[MAXHOST];
        char vhost[MAXHOST];
        char pass[MAXPASS];
        char desc[LINE];
        unsigned int flags;		
        char html[BUFSIZE];  
        int html_time;
	char dump_file[BUFSIZE];
	int dump_time;
        char join_chans[BUFSIZE];
        char url[BUFSIZE];
        int flood;
        int flood_time;
	unsigned int default_flags;
        char netname[MAXNICK + 1];  
        char admin_name[LINE];
        char admin_email[LINE];
} settings;	

struct me
{
	time_t t_start;
	struct Server *s;
	dlink_list chan_list;
	unsigned int cmdflags;
#ifndef IRCD_2_10
	char *capabs;
#endif
} me;

#ifdef STATSERV
struct StatServ
{
	char nick[MAXNICK + 1];
	char user[MAXUSER];
	char host[MAXHOST];
	char rname[LINE];
} StatServ;
#endif

struct Server
{
	struct Server *next, *prev;
	char name[MAXHOST];
	char desc[LINE];
#ifdef TRACK_VERSION
	char version[BUFSIZE];
#endif
#ifdef TRACK_ADMIN
	char admin1[LINE];
	char admin2[LINE];
	char admin3[LINE];
#endif
	int hops;
#ifdef IRCD_2_10
	int token;
#endif
	long hash;
	struct Stats *stats;
	long users;
	int opers;
#if defined(IRCD_HYB7) || defined(IRCD_2_10) || defined(IRCD_BAHAMUT) || \
    defined(IRCD_HYBSVS)
	int away;
#endif
#ifdef IRCD_2_10
	int services;
#endif
	time_t connected_since;
#if defined(IRCD_HYB7) || defined(IRCD_BAHAMUT) || defined(IRCD_HYBSVS)
	time_t eob_time;
#endif
	time_t last_announce;
	unsigned int flags;
	struct Server *uplink;
};

struct User
{
	struct User *next, *prev;
	char nick[MAXNICK + 1];
	char hostname[MAXHOST];
	char username[MAXUSER];
	struct Server *server;
	struct TLD *tld;
	struct MyUser *myuser;
	int flood;
	time_t t_flood;
	long hash;
#ifdef OPER_MODES
	unsigned int operflags;
#endif
	unsigned int flags;
};

#ifdef IRCD_2_10
struct Service
{
	char nick[MAXNICK + 1];
        char server[MAXHOST];
        char dist[MAXHOST];
	char gecos[LINE];
};
#endif

struct Hidden
{
	char name[MAXHOST];
	int perm;
};

struct Connection
{
	char name[LINE];
	int fd;
	int flags;
	char hbuffer[BUFSIZE * 2];
	struct MyUser *myuser;
};

struct Stats
{
	struct Stats *next;
	char name[MAXHOST];
	int numsplits;
	long maxusers;
	time_t t_maxusers;
	int maxopers;
	time_t t_maxopers;
#ifdef IRCD_2_10
	int maxservices;
	time_t t_maxservices;
#endif
#if defined(IRCD_HYB7) || defined(IRCD_2_10) || defined(IRCD_BAHAMUT) || \
    defined(IRCD_HYBSVS)
	int maxaway;
	time_t t_maxaway;
#endif
	time_t lastseen;
	time_t starttime;
	unsigned int operkills;
	unsigned int serverkills;
};

struct daily
{
	int servers;
	time_t t_servers;
	long users;
	time_t t_users;
	int opers;
	time_t t_opers;
#ifdef IRCD_2_10
	int services;
	time_t t_services;
#endif
} daily;

struct TLD
{
	struct TLD *next;
	long hash;
	char tld[MAXTLD];
	char country[LINE];
	char native[LINE];
	int users;
};

struct MyUser
{
	struct MyUser *next;
	char username[MAXNICK + 1];
	char password[LINE];
	time_t lastseen;
	struct User *u;
	struct Connection *conn;
	unsigned int flags;
};

struct MsgTable
{
        char* name;
	long counter;
	int hash;
	struct MsgTable *next;
        int (*handler)();
};

#ifdef SPLITSERV

struct SpServer
{
        char name[MAXHOST];
        struct SpServer* uplink;
        int numleaf;
        time_t ts;
	dlink_list leaflist;
};

struct SplitServ
{
        char host[MAXHOST];
        char user[MAXUSER];
        char nick[MAXNICK + 1];
} SplitServ;

struct sp
{
        char uplink[MAXHOST];
        char desc[LINE];
        char version[LINE];
        int numserver;
        char join_chan[LINE];
        int delay;
        int text_time;
	long expire_time;
	unsigned int flags;
        char text_name[BUFSIZE];
	int map_setting;
	int split_setting;
	int dump_setting;
} sp;

struct mapreturn
{
	int count;
	int depth;
};

#endif

struct AdminServ
{
        char host[MAXHOST];
        char user[MAXUSER];
        char nick[MAXNICK + 1];
} AdminServ;

#ifdef JUPESERV

struct JupeServ
{
        char host[MAXHOST];
        char user[MAXUSER];
        char nick[MAXNICK + 1];
} JupeServ;

struct Jupe
{
       char jupe[MAXHOST];
       char oper[MAXNICK + 1];
       long ts;
       char reason[LINE];
};

#endif

struct HelpTable {
        char* name;
        char* filename;
        int myuonly;
};

#endif
