/* Stats - IRC Statistical Services for Hybrid, csircd, and IRCnet 2.10
** conf.c - Configuration file support functions
**
** Copyright W. Campbell and others.  See README for more details
** Some code Copyright: Jonathan George, Kai Seidler, ircd-hybrid Team,
**                      IRCnet IRCD developers.
**
** $Id: conf.c,v 1.72 2002/10/29 03:17:02 wcampbel Exp $
*/

#include "stats.h"
#include "struct.h"
#include "proto.h"
#include "token.h"
#include "flags.h"

static void AddChannel(char *chan);

struct ConfTable
{
	char *name;
	int rehashable;
	void (*handler)();
};

static void c_server_name(char *);
static void c_server_port(char *);
static void c_connect_to(char *);
static void c_connect_pass(char *);
static void c_use_vhost(char *);
static void c_vhost(char *);
static void c_server_desc(char *);
static void c_netname(char *);
static void c_admin_name(char *);
static void c_admin_email(char *);
static void c_reconnect_time(char *);
static void c_flood(char *);
static void c_flood_time(char *);
static void c_flood_ignore(char *);
static void c_lag_time(char *);
static void c_hide_server(char *);
static void c_statserv_user(char *);
static void c_statserv_host(char *);
static void c_statserv_nick(char *);
static void c_onlyopers(char *);
static void c_want_privmsg(char *);
static void c_lag_notices(char *);
static void c_notify_eob(char *);
static void c_enable_html(char *);
static void c_set_oper(char *);
static void c_silent(char *);
static void c_html_dump(char *);
static void c_html_time(char *);
static void c_channels(char *);
static void c_url(char *);
static void c_splitserv_desc(char *);
static void c_splitserv_delay(char *);
static void c_splitserv_host(char *);
static void c_splitserv_user(char *);
static void c_splitserv_nick(char *);
static void c_splitserv_host(char *);
static void c_splitserv_privmsg(char *);
static void c_splitserv_chanmsg(char *);
static void c_splitserv_enable_text(char *);
static void c_splitserv_enable_expire(char *);
static void c_splitserv_expire_time(char *);
static void c_splitserv_report(char *);
static void c_splitserv_text_time(char *);
static void c_splitserv_text_name(char *);
static void c_splitserv_map_setting(char *);
static void c_splitserv_split_setting(char *);
static void c_splitserv_dump_setting(char *);
static void c_splitserv_chan(char *);
static void c_splitserv_hide(char *);
static void c_adminserv_host(char *);
static void c_adminserv_nick(char *);
static void c_adminserv_user(char *);
static void c_adminserv_onlyopers(char *);
static void c_jupeserv_host(char *);
static void c_jupeserv_nick(char *);
static void c_jupeserv_user(char *);
static void c_jupeserv_persist(char *);
static void c_enable_dump(char *);
static void c_dump_file(char *);
static void c_dump_time(char *);
static void c_default_flags(char *);
static void c_allow_unauth_chat(char *);
static void c_dcc_oper_only(char *);

static struct ConfTable conf_table[] =
{
	{ "HIDE_SERVER",	1,	c_hide_server },
	{ "SERVER_NAME",	0,	c_server_name },
	{ "SERVER_PORT",	0,	c_server_port },
	{ "CONNECT_TO",		0,	c_connect_to },
	{ "CONNECT_PASS",	0,	c_connect_pass },
	{ "USE_VHOST",		0,	c_use_vhost },
	{ "VHOST",		0,	c_vhost },
	{ "SERVER_DESC",	0,	c_server_desc },
	{ "NETNAME",		1,	c_netname },
	{ "ADMIN_NAME",		1,	c_admin_name },
	{ "ADMIN_EMAIL",	1,	c_admin_email },
	{ "RECONNECT_TIME",	1,	c_reconnect_time },
	{ "FLOOD_TIME",		1,	c_flood_time },
	{ "FLOOD",		1,	c_flood },
	{ "FLOOD_IGNORE",	1,	c_flood_ignore },
	{ "LAG_TIME",		1,	c_lag_time },
	{ "STATSERV_HOST",	0,	c_statserv_host },
	{ "STATSERV_USER",	0,	c_statserv_user },
	{ "STATSERV_NICK",	0,	c_statserv_nick },
	{ "ONLYOPERS",		1,	c_onlyopers },
	{ "WANT_PRIVMSG",	1,	c_want_privmsg },
	{ "LAG_NOTICES",	1,	c_lag_notices },
	{ "NOTIFY_EOB",		1,	c_notify_eob },
	{ "ENABLE_HTML",	1,	c_enable_html },
	{ "ENABLE_DUMP",	1,	c_enable_dump },
	{ "SET_OPER",		0,	c_set_oper },
	{ "SILENT",		1,	c_silent },
	{ "HTML_DUMP",		1,	c_html_dump },
	{ "HTML_TIME",		1,	c_html_time },
	{ "DUMP_TIME",		1,	c_dump_time },
	{ "DUMP_FILE",		1,	c_dump_file },
	{ "CHANNELS",		0,	c_channels },
	{ "URL",		1,	c_url },
	{ "SPLITSERV_DESC",	0,	c_splitserv_desc },
	{ "SPLITSERV_DELAY",	1,	c_splitserv_delay },
	{ "SPLITSERV_HOST",	0,	c_splitserv_host },
	{ "SPLITSERV_USER",	0,	c_splitserv_user },
	{ "SPLITSERV_NICK",	0,	c_splitserv_nick },
	{ "SPLITSERV_PRIVMSG",	1,	c_splitserv_privmsg },
	{ "SPLITSERV_CHANMSG",	1,	c_splitserv_chanmsg },
	{ "SPLITSERV_ENABLE_TEXT",	1,	c_splitserv_enable_text },
	{ "SPLITSERV_ENABLE_EXPIRE",	1,	c_splitserv_enable_expire },
	{ "SPLITSERV_EXPIRE_TIME",	1,	c_splitserv_expire_time },
	{ "SPLITSERV_REPORT",	1,	c_splitserv_report },
	{ "SPLITSERV_TEXT_TIME",1,	c_splitserv_text_time },
	{ "SPLITSERV_TEXT_NAME",1,	c_splitserv_text_name },
	{ "SPLITSERV_MAP_SETTING",1,	c_splitserv_map_setting },
	{ "SPLITSERV_SPLIT_SETTING",1,    c_splitserv_split_setting },
	{ "SPLITSERV_DUMP_SETTING",1,    c_splitserv_dump_setting },
	{ "SPLITSERV_CHAN",	0,	c_splitserv_chan },
	{ "SPLITSERV_HIDE",	1,	c_splitserv_hide },
	{ "ADMINSERV_USER",	0,	c_adminserv_user },
	{ "ADMINSERV_HOST",	0,	c_adminserv_host },
	{ "ADMINSERV_NICK",	0,	c_adminserv_nick },
	{ "ADMINSERV_ONLYOPERS",1,	c_adminserv_onlyopers },
	{ "JUPESERV_HOST",	0,	c_jupeserv_host },
	{ "JUPESERV_USER",	0,	c_jupeserv_user },
	{ "JUPESERV_NICK",	0,	c_jupeserv_nick },
	{ "JUPESERV_PERSIST",	1,	c_jupeserv_persist },
	{ "DEFAULT_FLAGS",	1,	c_default_flags },
	{ "ALLOW_UNAUTH_CHAT",	1,	c_allow_unauth_chat },
	{ "DCC_OPER_ONLY",	1,	c_dcc_oper_only },
	{ (char *) 0,		0,	(void (*)()) 0 }
};

#ifdef SPLITSERV
static struct Token split_options[] =
{
	{ "OPEN",	SO_OPEN },
	{ "OPERONLY",	SO_OPERONLY },
	{ "DISABLED",	SO_DISABLED },
	{ NULL,		TOKEN_ERROR }
};
#endif

/* Remove CR and LF from the line input.  ASSUME that the line 
** does NOT contain CR or LF in the middle
*/
void strip(char *line)
{
	char *c;
	int len;

	if ((c = strchr(line, '\r')))
	{
		/* If we receive a CR, then we can just terminate the
		** call there.
		**
		** Not any longer :(  - We have to pare off the spaces
		** at the end of lines
		*/
		*c = '\0';
	}

	if ((c = strchr(line, '\n')))
	{
		*c = '\0';
	}

	/* Strip out trailing spaces, which can cause odd issues with
	** SplitServ's channel and other things
	*/
	len = strlen(line);
	while (len > 0)
	{
		if (line[len - 1] == ' ')
		{
			line[len - 1] = '\0';
			len--;
		}
		else
		{
			break;
		}
	}

	return;
}

/* Set default parameters.  The settings used are the same as
** what is provided in stats.cfg.dist except for the binary
** ones, which MUST be default 0 in order for the conf load
** to work properly
*/
void InitDefaults()
{
	/* These strcpy's are safe, as they are static strings */

	/* All bit values must be initialized to 0 */
	strcpy(settings.name, "stats.network.tld");
	settings.port = 6666;
	strcpy(settings.uplink, "hub.network.tld");
	strcpy(settings.vhost, "0.0.0.0");
	strcpy(settings.pass, "password");
	strcpy(settings.desc, "Network Statistical Services");
	settings.flags=0x00000000; /* 0 is fine, but be explicit */
	settings.r_time = 10;
	settings.flood = 6;
	settings.flood_time = 10;
#ifdef STATSERV
	strcpy(StatServ.host, "stats.network.tld");
	strcpy(StatServ.user, "stats");
	strcpy(StatServ.nick, "StatServ");
#endif
	strcpy(settings.html, "/home/user/public_html/stats.html");
	settings.html_time = 3600;
	strcpy(settings.dump_file, "dump.txt");
	settings.dump_time = 3600;
	strcpy(settings.join_chans, "#statserv,#opers");
	strcpy(settings.url, "http://www.network.tld/stats.html");
	strcpy(settings.netname, "Misconfigured");
	strcpy(settings.admin_name, "Misconfigured Admin (mis-nick)");
	strcpy(settings.admin_email, "admin@misconfigured.net");
	/* The chan_list is derived from the join_chans...don't set it */
#ifdef SPLITSERV
	strcpy(sp.desc, "Split Services");
	sp.delay = 15;
	strcpy(SplitServ.host, "split.network.tld");
	strcpy(SplitServ.user, "split");
	strcpy(SplitServ.nick, "SplitServ");
	sp.flags = 0;
	sp.text_time = 1800;
	sp.expire_time = 172800L;
	strcpy(sp.text_name, "map.txt");
	strcpy(sp.join_chan, "#split");
	sp.dump_setting = SO_DISABLED;
	/* The other split settings default to OPEN */
#endif
	strcpy(AdminServ.nick, "AdminServ");
	strcpy(AdminServ.user, "admin");
	strcpy(AdminServ.host, "admin.network.tld");
#ifdef JUPESERV
	strcpy(JupeServ.nick, "JupeServ");
	strcpy(JupeServ.user, "jupe");
	strcpy(JupeServ.host, "jupe.network.tld");
#endif
	settings.default_flags = (CF_NOTICES|CF_CHAT);
}

/* Used to verify setting from the conf during startup and rehash...
** This prevents invalid variables from being used
*/
void VerifyConf()
{
        /* Sanity checking for FLOOD */
        if (settings.flood == 1)
        {
                settings.flood = 2;
        }
}

void ConfLoad()
{
        FILE *fp;
        char buf[BUFSIZE];
	char *cmd = buf;
        char *param;
	int i, found;

        if ((fp = fopen(CONFIG_NAME, "r")) == NULL)
        {
                printf("Unable to open configuration file (%s)\n", CONFIG_NAME);
                exit(0);
        }

        while (fgets(buf, BUFSIZE, fp))
	{
                if (buf[0] == '#')
                        continue;
		strip(buf);
		if (buf[0] == '\0')
			continue;
		param = strchr(buf, ' ');
		if (param)
		{
			*param = '\0';
			param++;
		}
		else
		{
			param = NULL;
		}
		found = 0;
		
		for (i=0; conf_table[i].name; i++)
		{
			/* using strncasecmp will be bad here, consider
			** "FLOOD"
			*/
			if (!strcasecmp(cmd, conf_table[i].name))
			{
				conf_table[i].handler(param);
				found = 1;
				break;
			}
		}

		if (!found)
			log("Invalid conf file option: %s", cmd);
	}

	fclose(fp);
}

static void c_server_name(char *param)
{
	if (param)
	{
		strlcpy(settings.name, param, sizeof(settings.name));
	}
	else
	{
		log("Invalid parameter for SERVER_NAME");
	}
}

static void c_hide_server(char *param)
{
	if (param)
	{
		AddHidden(param, 1);
	}
	else
	{
		log("Invalid parameter for HIDE_SERVER");
	}
}

static void c_server_port(char *param)
{
        if (param)
        {
		settings.port = atoi(param);
        }
        else
        {
                log("Invalid parameter for SERVER_PORT");
        }
}

static void c_connect_to(char *param)
{
        if (param)
	{
		/* Jeremy found that [ipv6-ip] will core it (enclosing
		** the IPv6 IP in [] seems to be a "standard" of IRC
		** client programs on UNIX.  Extra processing here is to
		** make sure it doesn't core...
		**
		** This is sub-optimal...it should run through the
		** param string and verify every character...
		*/
		if (param[0] == '[')
		{
			int len;
			param++;
			len = strlen(param);
			if ((len > 0) && (param[len - 1] == ']'))
				param[len - 1] = '\0';
			log("WARNING:  Brackets [] used in CONNECT_TO");
		}
			
                strlcpy(settings.uplink, param, sizeof(settings.uplink));
        }
        else
        {
                log("Invalid parameter for CONNECT_TO");
        }
}

static void c_connect_pass(char *param)
{
        if (param)
	{
                strlcpy(settings.pass, param, sizeof(settings.pass));
        }
        else
        {
                log("Invalid parameter for CONNECT_PASS");
        }
}

static void c_use_vhost(char *param)
{
	settings.flags |= FLAG_VHOST;
}

static void c_vhost(char *param)
{
        if (param)
	{
                /* Jeremy's [ipv6-ip] core, look above for more info */
                if (param[0] == '[')
                {
                        int len;
                        param++;
                        len = strlen(param);
                        if ((len > 0) && (param[len - 1] == ']'))
                                param[len - 1] = '\0';
                        log("WARNING:  Brackets [] used in VHOST");
                }

                strlcpy(settings.vhost, param, sizeof(settings.vhost));
        }
        else
        {
                log("Invalid parameter for VHOST");
        }
}

static void c_server_desc(char *param)
{
        if (param)
	{
                strlcpy(settings.desc, param, sizeof(settings.desc));
        }
        else
        {
                log("Invalid parameter for SERVER_DESC");
        }
}

static void c_netname(char *param)
{
        if (param)
	{
                strlcpy(settings.netname, param, sizeof(settings.netname));
        }
        else
        {
                log("Invalid parameter for NETNAME");
        }
}

static void c_admin_name(char *param)
{
        if (param)
	{
                strlcpy(settings.admin_name, param,
			sizeof(settings.admin_name));
        }
        else
        {
                log("Invalid parameter for ADMIN_NAME");
        }
}

static void c_admin_email(char *param)
{
        if (param)
	{
                strlcpy(settings.admin_email, param,
			sizeof(settings.admin_email));
        }
        else
        {
                log("Invalid parameter for ADMIN_EMAIL");
        }
}

static void c_reconnect_time(char *param)
{
        if (param)
	{
		settings.r_time = atoi(param);
        }
        else
        {
                log("Invalid parameter for RECONNECT_TIME");
        }
}

static void c_flood(char *param)
{
        if (param)
	{
		settings.flood = atoi(param);
        }
        else
        {
                log("Invalid parameter for FLOOD");
        }
}

static void c_flood_time(char *param)
{
        if (param)
	{
		settings.flood_time = atoi(param);
        }
        else
        {
                log("Invalid parameter for FLOOD_TIME");
        }
}

static void c_flood_ignore(char *param)
{
	settings.flags |= FLAG_FLOOD;
}

static void c_lag_time(char *param)
{
	/* XXX - obsolete option */
}

static void c_statserv_host(char *param)
{
#ifdef STATSERV
        if (param)
	{
                strlcpy(StatServ.host, param, sizeof(StatServ.host));
        }
        else
        {
                log("Invalid parameter for STATSERV_HOST");
        }
#endif
}

static void c_statserv_user(char *param)
{
#ifdef STATSERV
        if (param)
	{
                strlcpy(StatServ.user, param, sizeof(StatServ.user));
        }
        else
        {
                log("Invalid parameter for STATSERV_USER");
        }
#endif
}

static void c_statserv_nick(char *param)
{
#ifdef STATSERV
        if (param)
	{
                strlcpy(StatServ.nick, param, sizeof(StatServ.nick));
        }
        else
        {
                log("Invalid parameter for STATSERV_NICK");
        }
#endif
}

static void c_onlyopers(char *param)
{
	settings.flags |= FLAG_ONLYOPERS;
}

static void c_adminserv_onlyopers(char *param)
{
	settings.flags |= FLAG_AONLYOPERS;
}

static void c_want_privmsg(char *param)
{
	settings.flags |= FLAG_PRIVMSG;
}

static void c_lag_notices(char *param)
{
	/* XXX - obsolete option */
}

static void c_notify_eob(char *param)
{
#if defined(IRCD_HYB7) || defined(IRCD_BAHAMUT) || defined(IRCD_HYBSVS)
	settings.flags |= FLAG_NOTIFYEOB;
#else
	/* Nothing */
	;
#endif
}

static void c_enable_html(char *param)
{
	settings.flags |= FLAG_HTML;
}

static void c_enable_dump(char *param)
{
	settings.flags |= FLAG_DUMP;
}

static void c_set_oper(char *param)
{
	settings.flags |= FLAG_SETOPER;
}

static void c_silent(char *param)
{
        settings.flags |= FLAG_SILENT;
}

static void c_html_dump(char *param)
{
        if (param)
        {
                strlcpy(settings.html, param, sizeof(settings.html));
        }
        else
        {
                log("Invalid parameter for HTML_DUMP");
        }
}

static void c_html_time(char *param)
{
        if (param)
        {
		settings.html_time = atoi(param);
        }
        else
        {
                log("Invalid parameter for HTML_TIME");
        }
}

static void c_dump_file(char *param)
{
        if (param)
        {
                strlcpy(settings.dump_file, param, sizeof(settings.dump_file));
        }
        else
        {
                log("Invalid parameter for DUMP_FILE");
        }
}

static void c_dump_time(char *param)
{
        if (param)
        {
                settings.dump_time = atoi(param);
        }
        else
        {
                log("Invalid parameter for DUMP_TIME");
        }
}

static void c_channels(char *param)
{
        if (param)
        {
		AddChannel(param);
        }
        else
        {
                log("Invalid parameter for CHANNELS");
        }
}

static void c_url(char *param)
{
        if (param)
        {
                strlcpy(settings.url, param, sizeof(settings.url));
        }
        else
        {
                log("Invalid parameter for URL");
        }
}

static void c_splitserv_desc(char *param)
{
#ifdef SPLITSERV
        if (param)
        {
                strlcpy(sp.desc, param, sizeof(sp.desc));
        }
        else
        {
                log("Invalid parameter for SPLITSERV_DESC");
        }
#endif
}

static void c_splitserv_delay(char *param)
{
#ifdef SPLITSERV
        if (param)
        {
		sp.delay = atoi(param);
        }
        else
        {
                log("Invalid parameter for SPLITSERV_DELAY");
        }
#endif
}

static void c_splitserv_host(char *param)
{
#ifdef SPLITSERV
        if (param)
        {
                strlcpy(SplitServ.host, param, sizeof(SplitServ.host));
        }
        else
        {
                log("Invalid parameter for SPLITSERV_HOST");
        }
#endif
}

static void c_splitserv_user(char *param)
{
#ifdef SPLITSERV
        if (param)
        {
                strlcpy(SplitServ.user, param, sizeof(SplitServ.user));
        }
        else
        {
                log("Invalid parameter for SPLITSERV_USER");
        }
#endif
}

static void c_splitserv_nick(char *param)
{
#ifdef SPLITSERV
        if (param)
        {
                strlcpy(SplitServ.nick, param, sizeof(SplitServ.nick));
        }
        else
        {
                log("Invalid parameter for SPLITSERV_NICK");
        }
#endif
}

static void c_splitserv_privmsg(char *param)
{
#ifdef SPLITSERV
	sp.flags |= SPF_PRIVMSG;
#endif
}

static void c_splitserv_chanmsg(char *param)
{
#ifdef SPLITSERV
	sp.flags |= SPF_CHANMSG;
#endif
}

static void c_splitserv_hide(char *param)
{
#ifdef SPLITSERV
	sp.flags |= SPF_HIDE;
#endif
}
static void c_splitserv_enable_text(char *param)
{
#ifdef SPLITSERV
        sp.flags |= SPF_ENABLE_TEXT;
#endif
}

static void c_splitserv_enable_expire(char *param)
{
#ifdef SPLITSERV
	sp.flags |= SPF_EXPIRE;
#endif
}

static void c_splitserv_report(char *param)
{
#ifdef SPLITSERV
        sp.flags |= SPF_REPORT;
#endif
}

static void c_splitserv_text_time(char *param)
{
#ifdef SPLITSERV
        if (param)
        {
		sp.text_time = atoi(param);
        }
        else
        {
                log("Invalid parameter for SPLITSERV_TEXT_TIME");
        }
#endif
}

static void c_splitserv_expire_time(char *param)
{
#ifdef SPLITSERV
        if (param)
        {
                sp.expire_time = atol(param);
        }
        else
        {
                log("Invalid parameter for SPLITSERV_EXPIRE_TIME");
        }
#endif
}

static void c_splitserv_text_name(char *param)
{
#ifdef SPLITSERV
        if (param)
        {
                strlcpy(sp.text_name, param, sizeof(sp.text_name));
        }
        else
        {
                log("Invalid parameter for SPLITSERV_TEXT_NAME");
        }
#endif
}

static void c_splitserv_chan(char *param)
{
#ifdef SPLITSERV
        if (param)
        {
                strlcpy(sp.join_chan, param, sizeof(sp.join_chan));
        }
        else
        {
                log("Invalid parameter for SPLITSERV_CHAN");
        }
#endif
}

static void c_splitserv_map_setting(char *param)
{
#ifdef SPLITSERV
	if (param != NULL)
	{
		int value = token_to_value(split_options, param);
		if ((value == TOKEN_UNMATCHED) || (value == TOKEN_ERROR))
		{
			log("Invalid parameter for SPLITSERV_MAP_SETTING");
			return;
		}
		sp.map_setting = value;
	}
	else
	{
		log("Invalid parameter for SPLITSERV_MAP_SETTING");
	}
#endif
}

static void c_splitserv_split_setting(char *param)
{
#ifdef SPLITSERV
        if (param != NULL)
        {
                int value = token_to_value(split_options, param);
                if ((value == TOKEN_UNMATCHED) || (value == TOKEN_ERROR))
                {
                        log("Invalid parameter for SPLITSERV_SPLIT_SETTING");
                        return;
                }
                sp.split_setting = value;
        }
        else
        {
                log("Invalid parameter for SPLITSERV_SPLIT_SETTING");
        }
#endif
}

static void c_splitserv_dump_setting(char *param)
{
#ifdef SPLITSERV
        if (param != NULL)
        {
                int value = token_to_value(split_options, param);
                if ((value == TOKEN_UNMATCHED) || (value == TOKEN_ERROR))
                {
                        log("Invalid parameter for SPLITSERV_DUMP_SETTING");
                        return;
                }
                sp.dump_setting = value;
        }
        else
        {
                log("Invalid parameter for SPLITSERV_DUMP_SETTING");
        }
#endif
}

static void c_adminserv_user(char *param)
{
        if (param)
        {
                strlcpy(AdminServ.user, param, sizeof(AdminServ.user));
        }
        else
        {
                log("Invalid parameter for ADMINSERV_USER");
        }
}

static void c_adminserv_host(char *param)
{
        if (param)
        {
                strlcpy(AdminServ.host, param, sizeof(AdminServ.host));
        }
        else
        {
                log("Invalid parameter for ADMINSERV_HOST");
        }
}

static void c_adminserv_nick(char *param)
{
        if (param)
        {
                strlcpy(AdminServ.nick, param, sizeof(AdminServ.nick));
        }
        else
        {
                log("Invalid parameter for ADMINSERV_NICK");
        }
}

static void c_jupeserv_user(char *param)
{
#ifdef JUPESERV
	if (param)
	{
		strlcpy(JupeServ.user, param, sizeof(JupeServ.user));
	}
	else
	{
		log("Invalid parameter for JUPESERV_USER");
	}
#endif
}

static void c_jupeserv_host(char *param)
{
#ifdef JUPESERV
        if (param)
        {
                strlcpy(JupeServ.host, param, sizeof(JupeServ.host));
        }
        else
        {
                log("Invalid parameter for JUPESERV_HOST");
        }
#endif
}

static void c_jupeserv_nick(char *param)
{
#ifdef JUPESERV
        if (param)
        {
                strlcpy(JupeServ.nick, param, sizeof(JupeServ.nick));
        }
        else
        {
                log("Invalid parameter for JUPESERV_NICK");
        }
#endif
}

static void c_jupeserv_persist(char *param)
{
#ifdef JUPESERV
	settings.flags |= FLAG_JUPE_PERSIST;
#endif
}

static void c_default_flags(char *param)
{
	int parc;
	char *parv[MAXPARA];
	int i;
	int val;

        /* Clear the parv */
        for (i = 0; i < MAXPARA; i++)
                parv[i] = NULL;

        if (param)
        {
		parc = msgtoken(param, parv);
		for (i = 0; i < parc; i++)
		{
			val = token_to_value(flags_table, parv[i]);
			if ((val != TOKEN_UNMATCHED) && (val != TOKEN_ERROR))
			{
				settings.default_flags |= val;
			}
			else
			{
				log("DEFAULT_FLAGS: Unknown flag: %s", parv[i]);
			}
		}
        }
        else
        {
                log("Invalid parameter for DEFAULT_FLAGS");
        }
}

static void c_allow_unauth_chat(char *param)
{
	settings.flags |= FLAG_UNAUTHCHAT;
}

static void c_dcc_oper_only(char *param)
{
	settings.flags |= FLAG_DCCOPERONLY;
}

static void AddChannel(char *chan)
{
	char *s = NULL;
	char *pos = chan;
	dlink_node *dl;

	while ( (s = strchr(pos, ',')) )
	{
		*s = '\0';
		s++;
		if (*s == ' ')
			s++;
		dl = smalloc(sizeof(dlink_node));
		dlink_add(strdup(pos), dl, &me.chan_list);
		pos = s;
	}

	if (pos && (*pos != '\n'))
	{
		dl = smalloc(sizeof(dlink_node));
		dlink_add(strdup(pos), dl, &me.chan_list);
	}
}

void rehash()
{
	/* Should I free and reload the TLD file here? */

        FILE *fp;
        char buf[BUFSIZE];
        char *cmd = buf;
        char *param;
        int i, found;

	/* First things first...reset the bit flags to 0 */
	settings.flags = 0;
#ifdef SPLITSERV
	sp.flags = 0;
#endif

	/* Flush the hidden servers list */
	while (hiddenhead.head)
	{
		struct Hidden *h = hiddenhead.head->data;
		dlink_node *dl = hiddenhead.head;
		dlink_delete(hiddenhead.head, &hiddenhead);
		free(h);
		free(dl);
	}

	/* Weee, done the preliminary stuff */

        if ((fp = fopen(CONFIG_NAME, "r")) == NULL)
        {
                printf("Unable to open configuration file (%s)\n", CONFIG_NAME);
                exit(0);
        }

        while (fgets(buf, BUFSIZE, fp))
        {
                if (!buf[0])
                        continue;
                if (buf[0] == '#')
                        continue;
		strip(buf);
		if (buf[0] == '\0')
			continue;
                param = strchr(buf, ' ');
                if (param)
                {
                        *param = '\0';
                        param++;
                }
                else
                {
                        param = NULL;
                }
                found = 0;

                for (i=0; conf_table[i].name; i++)
                {
                        /* using strncasecmp will be bad here, consider
                        ** "FLOOD"
                        */
                        if (!strcasecmp(cmd, conf_table[i].name))
                        {
				if (conf_table[i].rehashable)
                                	conf_table[i].handler(param);
                                found = 1;
                                break;
                        }
                }

                if (!found)
                        log("Invalid conf file option: %s", cmd);
        }

        fclose(fp);

	VerifyConf();

	/* And finally (just in case the times changed) reset the
	** conf time based events
	*/
	ResetTimedEvents();
}

