/* Stats - IRC Statistical Services for Hybrid, csircd, and IRCnet 2.10
** datadump.c - Raw data dump function
**
** Copyright W. Campbell and others.  See README for more details
** Some code Copyright: Jonathan George, Kai Seidler, ircd-hybrid Team,
**                      IRCnet IRCD developers.
**
** $Id: datadump.c,v 1.7 2002/10/29 03:17:02 wcampbel Exp $
*/

#include "stats.h"
#include "struct.h"
#include "proto.h"

void datadump(char *filename)
{
	FILE *fp = fopen(filename, "w");
	time_t now = time(NULL);
	struct Server *s;
	int i;

	if (!fp)
	{
		log("Unable to open data dump file %s", filename);
		return;
	}

	fprintf(fp, "NETINFO %u %ld %ld %ld %d %d %ld %d %d %ld %d %d "
		"%ld %d %ld\n",
		netinfo.away, netinfo.users, netinfo.maxusers,
		netinfo.t_maxusers, netinfo.opers, netinfo.maxopers,
		netinfo.t_maxopers, netinfo.servers, netinfo.maxservers,
		netinfo.t_maxservers,
#ifdef IRCD_2_10
		netinfo.services, netinfo.maxservices, netinfo.t_maxservices,
		netinfo.maxaway, netinfo.t_maxaway);
#elif defined(IRCD_HYB7) || defined(IRCD_HYB7)
		0, 0, 0L, netinfo.maxaway, netinfo.t_maxaway);
#else
		0, 0, 0L, 0, 0L);
#endif

	fprintf(fp, "DAILY %ld %ld %ld %d %ld %d %ld %d %ld\n", now,
		daily.users, daily.t_users, daily.opers, daily.t_opers,
		daily.servers, daily.t_servers,
#ifdef IRCD_2_10
		daily.services, daily.t_services);
#else
		0, 0L);
#endif

	for (i = 0; i < S_TABLE_SIZE; i++)
	{
		for (s = serverlist[i]; s; s = s->next)
		{
		    fprintf(fp,
			"%s %d %ld %ld %ld %d %d %ld %d %u %u %d %d %ld "
			"%d %d %ld\n",
			s->name, IsSHide(s) ? 1 : 0, s->users,
			s->stats->maxusers, s->stats->t_maxusers, s->opers,
			s->stats->maxopers, s->stats->t_maxopers,
			s->stats->numsplits, s->stats->operkills,
			s->stats->serverkills,
#ifdef IRCD_2_10
			s->services, s->stats->maxservices,
			s->stats->t_maxservices, s->away, s->stats->maxaway,
			s->stats->t_maxaway);
#elif defined(IRCD_HYB7) || defined(IRCD_BAHAMUT) || defined(IRCD_HYBSVS)
			0, 0, 0L, s->away, s->stats->maxaway,
			s->stats->t_maxaway);
#else
			0, 0, 0L, 0, 0, 0L);
#endif
		}
	}

	fclose(fp);
}
