/* Stats - IRC Statistical Services for Hybrid, csircd, and IRCnet 2.10
** dcc.c - DCC handling
**
** Copyright W. Campbell and others.  See README for more details
** Some code Copyright: Jonathan George, Kai Seidler, ircd-hybrid Team,
**                      IRCnet IRCD developers.
**
** $Id: dcc.c,v 1.36 2002/10/30 20:20:19 wcampbel Exp $
*/

#include "stats.h"
#include "struct.h"
#include "proto.h"
#include "setup.h"
#include "token.h"
#include "flags.h"
#include "event.h"

#ifdef HAVE_CRYPT_H
#include <crypt.h>
#endif

static char errstr[BUFSIZE];

static int d_whom(struct Connection *, int, char **);
static int d_help(struct Connection *, int, char **);
static int d_login(struct Connection *, int, char **);
static int d_echo(struct Connection *, int, char **);
static int d_flags(struct Connection *, int, char **);
static int d_rehash(struct Connection *, int, char **);
static int d_restart(struct Connection *, int, char **);
static int d_shutdown(struct Connection *, int, char **);
static int d_quit(struct Connection *, int, char **);
static int d_user(struct Connection *, int, char **);
static int d_setpass(struct Connection *, int, char **);
static int d_quote(struct Connection *, int, char **);
static int d_jupe(struct Connection *, int, char **);
static int d_unjupe(struct Connection *, int, char **);
static int d_stats(struct Connection *, int, char **);
static int d_htmlupdate(struct Connection *, int, char **);
static int d_spforget(struct Connection *, int, char **);
static int d_spadd(struct Connection *, int, char **);
static int d_mapupdate(struct Connection *, int, char **);
static int d_kick(struct Connection *, int, char **);
static int d_me(struct Connection *, int, char **);
static int d_events(struct Connection *, int, char **);

static struct MsgTable dcc_table[] =
{
	{ ".whom",	0,0,NULL,	d_whom },
	{ ".help",	0,0,NULL,	d_help },
	{ ".login",	0,0,NULL,	d_login },
	{ ".echo",	0,0,NULL,	d_echo },
	{ ".flags",	0,0,NULL,	d_flags },
	{ ".rehash",	0,0,NULL,	d_rehash },
	{ ".restart",	0,0,NULL,	d_restart },
	{ ".die",	0,0,NULL,	d_shutdown },
	{ ".shutdown",	0,0,NULL,	d_shutdown },
	{ ".quit",	0,0,NULL,	d_quit },
	{ ".user",	0,0,NULL,	d_user },
	{ ".setpass",	0,0,NULL,	d_setpass },
	{ ".quote",	0,0,NULL,	d_quote },
	{ ".raw",	0,0,NULL,	d_quote },
	{ ".jupe",	0,0,NULL,	d_jupe },
	{ ".unjupe",	0,0,NULL,	d_unjupe },
	{ ".stats",	0,0,NULL,	d_stats },
	{ ".htmlupdate",0,0,NULL,	d_htmlupdate },
	{ ".spforget",	0,0,NULL,	d_spforget },
	{ ".spadd",	0,0,NULL,	d_spadd },
	{ ".mapupdate",	0,0,NULL,	d_mapupdate },
	{ ".kick",	0,0,NULL,	d_kick },
	{ ".me",	0,0,NULL,	d_me },
	{ "\001ACTION",	0,0,NULL,	d_me },
	{ ".events",	0,0,NULL,	d_events },
	{ (char *)0,	0,0,NULL,	(int (*)()) 0 }
};

static struct HelpTable help_table[] =
{
	{ "whom",	"help/dcc/whom",	0 },
	{ "login",	"help/dcc/login",	0 },
	{ "echo",	"help/dcc/echo",	0 },
	{ "flags",	"help/dcc/flags",	1 },
	{ "rehash",	"help/dcc/rehash",	1 },
	{ "restart",	"help/dcc/restart",	1 },
	{ "die",	"help/dcc/shutdown",	1 },
	{ "shutdown",	"help/dcc/shutdown",	1 },
	{ "quit",	"help/dcc/quit",	0 },
	{ "user",	"help/dcc/user",	1 },
	{ "setpass",	"help/dcc/setpass",	1 },
	{ "jupe",	"help/dcc/jupe",	1 },
	{ "unjupe",	"help/dcc/unjupe",	1 },
	{ "stats",	"help/dcc/stats",	1 },
	{ "htmlupdate",	"help/dcc/htmlupdate",	1 },
	{ "spforget",	"help/dcc/spforget",	1 },
	{ "spadd",	"help/dcc/spadd",	1 },
	{ "mapupdate",	"help/dcc/mapupdate",	1 },
	{ "kick",	"help/dcc/kick",	1 },
	{ "me",		"help/dcc/me",		0 },
	{ "events",	"help/dcc/events",	1 },
	{ (char *)0,	(char *)0,		0 }
};

void dcc_parse(int fd, char *line)
{
        char *parv[MAXPARA];
        int parc = 0;
        int i;
        int status;
        static char buf[BUFSIZE];
	struct Connection *conn;

        /* Clear the parv */
        for (i = 0; i < MAXPARA; i++)
                parv[i] = NULL;

        /* Clear the error string */
        errstr[0] = '\0';

        strlcpy(buf, line, BUFSIZE);

        if (line)
                parc = msgtoken(line, parv);

        netinfo.requests++;

        if (!parc)
                return;

	conn = find_connection(fd);

        /* All the preliminary junk is done, now we have to go through 
        ** the tables and call the handler for that command
        */
        for (i=0; dcc_table[i].handler; i++)
        {
                if (!strcasecmp(parv[0], dcc_table[i].name))
                {
                    status = dcc_table[i].handler(conn, parc, parv);
                    if (status == 1)
                        log("Error with dcc command %s: %s", parv[0],
                        (errstr) ? errstr : "No error message");
                    /* Matched a value, done, then return */
                    return;
                }
        }

	if (parv[0][0] == '.')
		sendto_dcc(fd, "Invalid command: %s", parv[0]);
	else
		if (conn->flags & CF_CHAT)
			sendto_partyline(conn, "From %s: %s", conn->name, buf);
		else
			sendto_dcc(fd, "You are not set +CHAT");
        return;
}

/* In all of these functions, parv[0] contains the command, followed
** by the parameters
*/

static int d_help(struct Connection *conn, int parc, char *parv[])
{
        int i;

        spy(".help Request by %s for %s", conn->name,
                (parv[1]) ? parv[1] : "NO QUERY");
	sendto_dcc_flag(CF_COMMANDS, ".help Request by %s for %s",
		conn->name, (parv[1]) ? parv[1] : "NO QUERY");
        if (parc < 2)
        {
                send_dcc_help_file(conn->fd, "help/dcc/index", "index");
                if (conn->myuser != NULL)
                        send_dcc_help_file(conn->fd, "help/dcc/aindex",
				"aindex");
                return 0;
        }

	/* Ignore any .'s at the beginning of the query */
	if (parv[1][0] == '.')
		parv[1]++;

        for (i = 0; help_table[i].filename; i++)
        {
                if (!strcasecmp(parv[1], help_table[i].name))
                {
                        if (help_table[i].myuonly)
                        {
                                if (conn->myuser)
                                        send_dcc_help_file(conn->fd,
                                                help_table[i].filename,
                                                help_table[i].name);
                                else
                                        sendto_dcc(conn->fd,
						"Permission Denied.");
                        }
                        else
                        {
                                send_dcc_help_file(conn->fd,
					help_table[i].filename,
                                        help_table[i].name);
                        }

                        return 0;
                }
        }

        sendto_dcc(conn->fd, "Unknown Help Topic: %s", parv[1]);
        snprintf(errstr, BUFSIZE, "Unknown HELP topic: %s", parv[1]);
        return 1;
}

static int d_echo(struct Connection *conn, int parc, char *parv[])
{
	if (parc < 2)
	{
		sendto_dcc(conn->fd, "Echo is %s", (conn->flags & CONN_NOECHO) ?
			"OFF" : "ON");
		return 0;
	}
	if (strcasecmp(parv[1], "ON") == 0)
	{
		conn->flags &= ~CONN_NOECHO;
		sendto_dcc(conn->fd, "Echo is now ON");
		return 0;
	}
	else if (strcasecmp(parv[1], "OFF") == 0)
	{
		conn->flags |= CONN_NOECHO;
		sendto_dcc(conn->fd, "Echo is now OFF");
		return 0;
	}
	else
	{
		sendto_dcc(conn->fd, "Invalid option to .echo, please use ON or OFF");
		/* user error */
		return 0;
	}

	/* NOT REACHED */
	return 0;
}

static int d_whom(struct Connection *conn, int parc, char *parv[])
{
	struct Connection *tc;
	dlink_node *dl;
	int users = 0;

	DLINK_FOREACH(dl, connection_list.head)
	{
		tc = (struct Connection *) dl->data;
		if (tc->flags & CONN_CONNECTING)
			continue;
		if (tc->flags & (CONN_DCCIN|CONN_DCCOUT))
			sendto_dcc(conn->fd, "User: %s [%s](fd: %d)", tc->name,
				(tc->myuser) ? tc->myuser->username : "*",
				tc->fd);
		users++;
	}

	if (users == 0)
	{
		/* If there's no users, how did we get this far? */
		strcpy(errstr, "No users in .whom");
		return 1;
	}

	return 0;
}

static int d_login(struct Connection *conn, int parc, char *parv[])
{
	struct MyUser *myu;
#if defined(CRYPT)
        char * crpass;
#endif

	if (parc < 3)
	{
		sendto_dcc(conn->fd,
			"Syntax: .login \37username\37 \37password\37");
		strcpy(errstr, "Missing parameters in .login");
		return 1;
	}

	if (conn == NULL)
	{
		strcpy(errstr, "Unknown connection in .login");
		return 1;
	}

	myu = findmyuser(parv[1]);
	if (myu == NULL)
	{
		alert("Failed .login attempt (unknown user: %s)", parv[1]);
		sendto_dcc_flag(CF_AUTH,
			"Failed .login attempt (unknown user: %s)", parv[1]);
		sendto_dcc(conn->fd, "Unknown User.");
		log("Failed login attempt (unknown user (%s))", parv[1]);
		/* Not a code error */
		return 0;
	}

#ifdef CRYPT
	crpass = crypt(parv[2], myu->password);
	if (!strcasecmp(myu->username, parv[1]) &&
		!strcmp(myu->password, crpass))
#else
	if (!strcasecmp(myu->username, parv[1]) &&
		!strcmp(myu->password, parv[2]))
#endif
	{
		log("%s [%s][fd: %d] successfully logged in.", conn->name,
			myu->username, conn->fd);
		alert("Successful .login by %s to myunick %s", conn->name,
			myu->username);
		sendto_dcc_flag(CF_AUTH,
			"Successful .login by %s to myunick %s", conn->name,
			myu->username);
		sendto_dcc(conn->fd, "Permission Granted.");
		sendto_all_dcc("%s [%s] successfully logged in.", conn->name,
			myu->username);
		conn->myuser = myu;
		myu->lastseen = time(NULL);
		myu->flags |= MFLAG_DCC;
		myu->conn = conn;
		conn->flags |= settings.default_flags;
		send_set_flags(conn);
		send_unset_flags(conn);
		return 0;
	}
	else
	{
		log("%s [%s][fd: %d] failed login.", conn->name,
			myu->username, conn->fd);
		alert("Failed .login attempt (invalid password) by %s on "
			"myuser nick %s", conn->name, myu->username);
		sendto_dcc_flag(CF_AUTH,
			"Failed .login attempt (invalid password) by %s on "
			"myuser nick %s", conn->name, myu->username);
		sendto_dcc(conn->fd, "Access Denied.");
		/* not a code error */
		return 0;
	}

	/* NOT REACHED */
	return 0;
}

static int d_flags(struct Connection *conn, int parc, char *parv[])
{
	int i;
	int val;
	char *tok;
	int add;

	if (conn->myuser == NULL)
	{
		sendto_dcc(conn->fd, "You must .login to use .flags");
		return 0;
	}

	if (parc < 2)
	{
		send_set_flags(conn);
		send_unset_flags(conn);
		return 0;
	}
	for (i = 1; i < parc; i++)
	{
		if (!isalpha((int) parv[i][0]))
		{
			if (parv[i][0] == '+')
				add = 1;
			else if (parv[i][0] == '-')
				add = 0;
			else
			{
				sendto_dcc(conn->fd,
					"Invalid type character: %c",
					parv[i][0]);
				continue;
			}

			tok = parv[i] + 1;
		}
		else
		{
			add = 1;
			tok = parv[i];
		}
		val = token_to_value(flags_table, tok);
		if ((val != TOKEN_UNMATCHED) && (val != TOKEN_ERROR))
		{
			if (add)
				conn->flags |= val;
			else
				conn->flags &= ~val;
		}
		else
		{
			sendto_dcc(conn->fd, "Unknown flag: %s", tok);
		}
	}
	send_set_flags(conn);
	send_unset_flags(conn);
	return 0;
}

static int d_rehash(struct Connection *conn, int parc, char *parv[])
{
	if (conn->myuser == NULL)
	{
		log("Access Denied (.rehash) to %s", conn->name);
		sendto_dcc(conn->fd, "Access Denied.");
		alert(".rehash Attempt by %s (failed)", conn->name);
		sendto_dcc_flag(CF_COMMANDS, ".rehash Attempt by %s (failed)",
			conn->name);
		/* Not a code error */
		return 0;
	}

	rehash();

	alert(".rehash by %s", conn->name);
	sendto_dcc_flag(CF_NOTICES, ".rehash by %s", conn->name);
	sendto_dcc(conn->fd, ".rehash complete");

	return 0;
}

static int d_restart(struct Connection *conn, int parc, char *parv[])
{
	char comm[BUFSIZE];

	if (conn->myuser == NULL)
	{
		log("Access Denied (.restart) to %s", conn->name);
		sendto_dcc(conn->fd, "Access Denied.");
		alert(".restart Attempt by %s (failed)", conn->name);
		sendto_dcc_flag(CF_COMMANDS, ".restart Attempt by %s (failed)",
			conn->name);
		/* Not a code error */
		return 0;
	}

	comm[0] = '\0';

	ExpandParv(comm, BUFSIZE, 1, parc, parv);

	wallops(settings.name, "%s requested RESTART: %s", conn->name,
		(comm[0]) ? comm : "No Reason");
	alert("Service Restart by %s: %s", conn->name,
		(comm[0]) ? comm : "No Reason");
	sendto_dcc_flag(CF_NOTICES, "Service Restart by %s: %s",
			conn->name, (comm[0]) ? comm : "No Reason");
#ifdef STATSERV
        sts(":%s QUIT :Restart Request by %s: %s", StatServ.nick, conn->name,
		(comm[0]) ? comm : "No Reason");
#endif
        sts(":%s QUIT :Restart Request by %s: %s", AdminServ.nick, conn->name,
		(comm[0]) ? comm : "No Reason");
#ifdef SPLITSERV
        sts(":%s QUIT :Restart Request by %s: %s", SplitServ.nick, conn->name,
		(comm[0]) ? comm : "No Reason");
#endif
#ifdef JUPESERV
	sts(":%s QUIT :Restart Request by %s: %s", JupeServ.nick, conn->name,
		(comm[0]) ? comm : "No Reason");
#endif
	sleep(1);
	close(servsock);
	close_dccs();
	SaveStats();
	log("%s [%s](fd %d) requested .restart: %s", conn->name,
		conn->myuser->username, conn->fd, (comm) ? comm : "No Reason");
	execv(SPATH, NULL);

	/* NOT REACHED */
	return 0;
}

static int d_shutdown(struct Connection *conn, int parc, char *parv[])
{
        char comm[BUFSIZE];

        if (conn->myuser == NULL)
        {
                log("Access Denied (.shutdown) to %s", conn->name);
                sendto_dcc(conn->fd, "Access Denied.");
                alert(".shutdown Attempt by %s (failed)", conn->name);
		sendto_dcc_flag(CF_COMMANDS, ".shutdown Attempt by %s (failed)",
			conn->name);
                /* Not a code error */
                return 0;
        }

        comm[0] = '\0';

        ExpandParv(comm, BUFSIZE, 1, parc, parv);

        wallops(settings.name, "%s requested SHUTDOWN: %s", conn->name,
                (comm[0]) ? comm : "No Reason");
        alert("Service Shutdown by %s: %s", conn->name,
                (comm[0]) ? comm : "No Reason");
	sendto_dcc_flag(CF_NOTICES, "Service Shutdown by %s: %s", conn->name,
		(comm[0]) ? comm : "No Reason");
#ifdef STATSERV
        sts(":%s QUIT :Shutdown Request by %s: %s", StatServ.nick, conn->name,
                (comm[0]) ? comm : "No Reason");
#endif
        sts(":%s QUIT :Shutdown Request by %s: %s", AdminServ.nick, conn->name,
                (comm[0]) ? comm : "No Reason");
#ifdef SPLITSERV
        sts(":%s QUIT :Shutdown Request by %s: %s", SplitServ.nick, conn->name,
                (comm[0]) ? comm : "No Reason");
#endif
#ifdef JUPESERV
        sts(":%s QUIT :Shutdown Request by %s: %s", JupeServ.nick, conn->name,
                (comm[0]) ? comm : "No Reason");
#endif
        sleep(1);
        close(servsock);
        close_dccs();
        SaveStats();
        log("%s [%s](fd %d) requested .shutdown: %s", conn->name,
                conn->myuser->username, conn->fd, (comm) ? comm : "No Reason");
	exit(0);

        /* NOT REACHED */
        return 0;
}

static int d_quit(struct Connection *conn, int parc, char *parv[])
{
	sendto_all_dcc("Client %s disconnected", conn->name);
	close(conn->fd);
	conn->flags |= CONN_DEAD; /* Just in case... */
	delete_connection(conn->fd);
	return 0;
}

static int d_user(struct Connection *conn, int parc, char *parv[])
{
	int a = 0;

	if (conn->myuser == NULL)
	{
		log("Denied access (.user) to %s [FD: %d].", conn->name,
			conn->fd);
		sendto_dcc(conn->fd, "Access Denied.");
		/* Not a code error */
		return 0;
	}

	if (parc < 2)
	{
		sendto_dcc(conn->fd, "Syntax: .user [add|del|list]");
		strcpy(errstr, "Missing parameter in .user");
		return 1;
	}

        if (!strcasecmp(parv[1], "LIST"))
                a = 0;
        else if (!strcasecmp(parv[1], "ADD"))
                a = 1;
        else if (!strcasecmp(parv[1], "DEL"))
                a = 2;
	else
	{
		sendto_dcc(conn->fd, "Unknown Flag: %s", parv[1]);
		snprintf(errstr, BUFSIZE, "Unknown flag: %s in USER", parv[1]);
		return 1;
	}

	switch (a)
	{
		case 0:
		{
			struct MyUser *myu;
			sendto_dcc(conn->fd, "UserList:");
			spy(".user list Request by %s", conn->name);
			sendto_dcc_flag(CF_COMMANDS,
				".user list Request by %s", conn->name);
			for (myu = myuhead; myu; myu = myu->next)
			{
				sendto_dcc(conn->fd, "%-20s (%s) -> %s",
					myu->username,
                                        (IsOperOn(myu)) ? "online" :
                                        ((IsOperDCC(myu)) ? "dcc" : "offline"),
					(myu->lastseen == 0) ? "Never Seen" :
					sftime(myu->lastseen));
			}
			sendto_dcc(conn->fd, "End of List.");
			break;
		}
		case 1:
		{
			struct MyUser *myu;
#if defined(CRYPT)
			char * salt;
#endif

			if (parc < 4)
			{
				sendto_dcc(conn->fd,
					"Syntax: .user add <username> "
					"<password>");
				strcpy(errstr, "Missing parameter in .user");
				return 1;
			}
			myu = smalloc(sizeof(struct MyUser));
			strlcpy(myu->username, parv[2], MAXNICK + 1);
#if defined(CRYPT)
			salt = make_salt();
			strlcpy(myu->password, crypt(parv[3], salt), LINE);
#else
			strlcpy(myu->password, parv[3], LINE);
#endif
			myu->lastseen = time(NULL);
			if (!myuhead)
			{
				myuhead = myu;
				myuhead->next = NULL;
			}
			else
			{
				myu->next = myuhead;
				myuhead = myu;
			}
			sendto_dcc(conn->fd, "Added user %s with password %s",
				parv[2], parv[3]);
			alert(".user add %s by %s", parv[2], conn->name);
			sendto_dcc_flag(CF_AUTH, ".user add %s by %s",
				parv[2], conn->name);
			log("%s [%s](FD: %d) added user %s", conn->name,
				conn->myuser->username, conn->fd,
				myu->username);
			SaveMyUsers();
			return 0;
			/* NOT REACHED */
			break;
		}
		case 2:
		{
			if (parc < 3)
			{
				sendto_dcc(conn->fd,
					"Syntax: .user del <username>");
				strcpy(errstr, "Missing parameter in .user");
				return 1;
			}
			if ((findmyuser(parv[2])) == NULL)
			{
				sendto_dcc(conn->fd, "Unable to find user %s",
					parv[2]);
				alert(".user del %s attempt by %s (failed)",
					parv[2], conn->name);
				sendto_dcc_flag(CF_COMMANDS,
					".user del %s attempt by %s (failed)",
					parv[2], conn->name);
				snprintf(errstr, BUFSIZE,
					"Unknown myuser: %s", parv[2]);
				return 1;
			}
			DeleteMyUser(parv[2]);
			SaveMyUsers();
			sendto_dcc(conn->fd, "Deleted user %s", parv[2]);
			alert(".user del %s by %s", parv[2], conn->name);
			sendto_dcc_flag(CF_AUTH, ".user del %s by %s",
				parv[2], conn->name);
			log("%s [%s](FD: %d) deleted user %s", conn->name,
				conn->myuser->username, conn->fd, parv[2]);
			return 0;
			/* NOT REACHED */
			break;
		}
	}

	/* NOT REACHED */
	return 0;
}
				
static int d_setpass(struct Connection *conn, int parc, char *parv[])
{
#if defined(CRYPT)
        char * salt;
#endif /* CRYPT */

	if (conn->myuser == NULL)
	{
		log("Access Denied (.setpass) to %s", conn->name);
		sendto_dcc(conn->fd, "Access Denied.");
		/* Not a code error */
		return 0;
	}

	if (parc < 2)
	{
		sendto_dcc(conn->fd, "Please enter a valid password");
		strcpy(errstr, "NULL password attempt in .setpass");
		return 1;
	}

#if defined(CRYPT)
        salt = make_salt();
        strlcpy(conn->myuser->password, crypt(parv[1], salt), LINE);
#else
        strlcpy(conn->myuser->password, parv[1], LINE);
#endif /* CRYPT */

	sendto_dcc(conn->fd, "Password changed to %s", parv[1]);
	log("%s (FD: %d) changed his/her password", conn->name, conn->fd);
	SaveMyUsers();

	return 0;
}

static int d_quote(struct Connection *conn, int parc, char *parv[])
{
#ifdef ENABLE_QUOTE
	char command[BUFSIZE];

	if (conn->myuser == NULL)
	{
		log("Access Denied (.quote) to %s", conn->name);
		sendto_dcc(conn->fd, "Access Denied.");
		/* Not an error in the code */
		return 0;
	}

	ExpandParv(command, BUFSIZE, 1, parc, parv);

	if (command[0] == '\0')
	{
		sendto_dcc(conn->fd, "Please enter a valid command");
		strcpy(errstr, "NULL command attempt in QUOTE");
		return 1;
	}

	sts("%s", command);
	log("%s (FD: %d) made me execute %s", conn->name, conn->fd, command);
	return 0;
#else
	sendto_dcc(conn->fd, ".quote/.raw Disabled.");
	return 0;
#endif
}

static int d_jupe(struct Connection *conn, int parc, char *parv[])
{
#ifdef JUPESERV
	struct Jupe *j;
	char reason[LINE];

	if (conn->myuser == NULL)
	{
		log("Access denied (.jupe) to %s", conn->name);
		sendto_dcc(conn->fd, "Access Denied.");
		spy(".jupe %s Attempt by %s (failed)",
			(parv[1]) ? parv[1] : "<nothing>", conn->name);
		sendto_dcc_flag(CF_JUPE, ".jupe %s Attempt by %s (failed)",
			(parv[1]) ? parv[1] : "<nothing>", conn->name);
		return 0;
	}

	if (parc < 2)
	{
		sendto_dcc(conn->fd, "Syntax:  .jupe <server> <reason>");
		spy(".jupe Attempt by %s (failed) (no parameter)", conn->name);
		sendto_dcc_flag(CF_JUPE,
			".jupe Attempt by %s (failed) (no parameter)",
			conn->name);
		strcpy(errstr, "Missing parameter in .jupe");
		return 1;
	}

	j = FindJupe(parv[1]);

	if (j != NULL)
	{
		sendto_dcc(conn->fd, "Cannot jupe already juped server");
		spy(".jupe %s Attempt by %s (already juped)", parv[1],
			conn->name);
		sendto_dcc_flag(CF_JUPE,
			".jupe Attempt by %s (failed) (no parameter)",
			conn->name);
		/* Not a code error */
		return 0;
	}

	if (strchr(parv[1], '.') == NULL)
	{
		sendto_dcc(conn->fd, "Only servers may be juped");
		spy(".jupe %s Attempt by %s (non-server)", parv[1],
			conn->name);
		sendto_dcc_flag(CF_JUPE, ".jupe %s Attempt by %s (non-server)",
			parv[1], conn->name);
		/* Not a code error */
		return 0;
	}

	if (strcasecmp(me.s->name, parv[1]) == 0)
	{
		sendto_dcc(conn->fd, "Cannot jupe your uplink");
		spy(".jupe %s Attempt by %s (uplink)", parv[1], conn->name);
		sendto_dcc_flag(CF_JUPE, ".jupe %s Attempt by %s (uplink)",
			parv[1], conn->name);
		/* Not a code error */
		return 0;
	}

	if (strcasecmp(settings.name, parv[1]) == 0)
	{
		sendto_dcc(conn->fd, "Cannot jupe myself");
		spy(".jupe %s Attempt by %s (myself)", parv[1], conn->name);
		sendto_dcc_flag(CF_JUPE, ".jupe %s Attempt by %s (myself)",
			parv[1], conn->name);
		/* Not a code error */
		return 0;
	}

	reason[0] = '\0';
	ExpandParv(reason, LINE, 2, parc, parv);
	if (reason[0] == '\0')
	{
		strcpy(reason, "No Reason");
	}

	AddJupe(parv[1], conn->myuser->username, time(NULL), reason);
	wallops(settings.name, "%s@fd:%d adding JUPE for %s - Reason: %s",
		conn->name, conn->fd, parv[1], reason);
	sendto_dcc(conn->fd, "Jupe %s added, enforced", parv[1]);
	alert(".jupe %s by %s: %s", parv[1], conn->name, reason);
	sendto_dcc_flag(CF_JUPE, ".jupe %s by %s: %s", parv[1], conn->name,
		reason);
	SaveJupes();

	return 0;
#else
	sendto_dcc(conn->fd, ".jupe Disabled.");
	return 0;
#endif
}

static int d_unjupe(struct Connection *conn, int parc, char *parv[])
{
#ifdef JUPESERV
	struct Jupe *j;

	if (conn->myuser == NULL)
	{
		log("Access denied (.unjupe) to %s", conn->name);
		spy(".unjupe %s Attempt by %s (failed)",
			(parv[1]) ? parv[1] : "<nothing>", conn->name);
		sendto_dcc_flag(CF_JUPE, ".unjupe %s Attempt by %s (failed)",
			(parv[1]) ? parv[1] : "<nothing>", conn->name);
		sendto_dcc(conn->fd, "Access Denied.");
		return 0;
	}

	if (parc < 2)
	{
		sendto_dcc(conn->fd, "Syntax:  .unjupe <server>");
		spy(".unjupe Attempt by %s (failed) (no parameter)",
			conn->name);
		sendto_dcc_flag(CF_JUPE,
			".unjupe Attempt by %s (failed) (no parameter)",
			conn->name);
		strcpy(errstr, "Missing parameter in .unjupe");
		return 0;
	}

	j = FindJupe(parv[1]);

	if (j == NULL)
	{
		sendto_dcc(conn->fd, "Jupe %s does not exist", parv[1]);
		spy(".unjupe %s Attempt by %s (jupe does not exist)",
			parv[1], conn->name);
		sendto_dcc_flag(CF_JUPE,
			".unjupe %s Attempt by %s (jupe does not exist)",
			parv[1], conn->name);
		/* Not a code error */
		return 0;
	}

	DelJupe(parv[1]);
	wallops(settings.name, "%s@fd:%d removing JUPE for %s", conn->name,
		conn->fd, parv[1]);
	alert(".unjupe %s by %s", parv[1], conn->name);
	sendto_dcc_flag(CF_JUPE, ".unjupe %s by %s", parv[1], conn->name);
	sendto_dcc(conn->fd, "Jupe %s removed", parv[1]);
	SaveJupes();

	return 0;
#else
	sendto_dcc(conn->fd, ".unjupe Disabled.");
	return 0;
#endif
}

static int d_stats(struct Connection *conn, int parc, char *parv[])
{
#ifdef STATSERV
	struct Stats *st;

	if (conn->myuser == NULL)
	{
		log("Access Denied (.stats) to %s", conn->name);
		sendto_dcc(conn->fd, "Access Denied.");
		/* Not a code error */
		return 0;
	}

	if (parc < 2)
	{
		sendto_dcc(conn->fd, "Syntax: .stats [del|list|copy]");
		strcpy(errstr, "Missing parameter in .stats");
		return 1;
	}

	if (!strcasecmp(parv[1], "LIST"))
	{
		int i = 1;
		spy(".stats list Request by %s", conn->name);
		sendto_dcc_flag(CF_COMMANDS, ".stats list Request by %s",
			conn->name);
		sendto_dcc(conn->fd, "Statistics Database:");
		for (st = shead; st; st = st->next)
		{
			sendto_dcc(conn->fd, "[%-2d] %s", i, st->name);
			i++;
		}
		sendto_dcc(conn->fd, "End of List.");
		log("%s requested .stats list", conn->name);
	}
	else if (!strcasecmp(parv[1], "DEL"))
	{
		struct Stats *m = NULL;
		if (parc < 3)
		{
			sendto_dcc(conn->fd, "Syntax: .stats del <name>");
			strcpy(errstr, "Missing parameter in .stats");
			return 1;
		}
		st = findstats(parv[2]);
		if (st == NULL)
		{
			spy(".stats del %s Attempt by %s (failed)",
				parv[2], conn->name);
			sendto_dcc_flag(CF_COMMANDS,
				".stats del %s Attempt by %s (failed)",
				parv[2], conn->name);
			sendto_dcc(conn->fd, "%s is not in the database!",
				parv[2]);
			snprintf(errstr, BUFSIZE, "Unknown server: %s",
				parv[2]);
			return 1;
		}
		for (st = shead; st; st = st->next)
		{
			if (!strcasecmp(parv[2], st->name))
				break;
			m = st;
		}
		if (m != NULL)
			m->next = st->next;
		else
			shead = st->next;
		free(st);
		spy(".stats del %s Request by %s", parv[2], conn->name);
		sendto_dcc_flag(CF_COMMANDS, ".stats del %s Request by %s",
			parv[2], conn->name);
		sendto_dcc(conn->fd, "Removed %s from the database.", parv[2]);
		log("%s requested .stats del %s", conn->name, parv[2]);
	}
	else if (!strcasecmp(parv[1], "COPY"))
	{
		struct Server *s;
		struct Stats *st2;
		struct Stats *m = NULL;
		struct Stats *found = NULL;

		if (parc < 4)
		{
			sendto_dcc(conn->fd, "Syntax: .stats copy <name> "
				"<newname>");
			strcpy(errstr, "Missing parameter in STATS COPY");
			return 1;
		}
		st = findstats(parv[2]);
		if (st == NULL)
		{
			sendto_dcc(conn->fd, "No entry in the database for %s",
				parv[2]);
			snprintf(errstr, BUFSIZE, "Unknown stats entry: %s",
				parv[2]);
			return 1;
		}
		s = findserver(parv[2]);
		if (s != NULL)
		{
			sendto_dcc(conn->fd, "Server %s is online!", parv[2]);
			snprintf(errstr, BUFSIZE, "Server %s is online",
				parv[2]);
			return 1;
		}

		/* Wait until we are sure before removing a statistics
		** entry
		*/
		for (st2 = shead; st2; st2 = st2->next)
		{
			if (!strcasecmp(parv[3], st2->name))
			{
				found = st2;
				break;
			}
			m = st2;
		}
		if (found != NULL)
		{
			if (m != NULL)
				m->next = found->next;
			else
				shead = found->next;
			free(found);
		}
		s = NULL;
		strlcpy(st->name, parv[3], sizeof(st->name));
		s = findserver(st->name);
		if (s != NULL)
			s->stats = st;
		sendto_dcc(conn->fd, "Moved database entry for %s to %s",
			parv[2], parv[3]);
		log("%s requested .stats copy %s -> %s", conn->name, parv[2],
			parv[3]);
		spy(".stats copy %s %s Request by %s", parv[2], parv[3],
			conn->name);
		sendto_dcc_flag(CF_COMMANDS, ".stats copy %s %s Request by %s",
			parv[2], parv[3], conn->name);
	}
	else
	{
		sendto_dcc(conn->fd, "Invalid Argument.");
		strcpy(errstr, "Invalid argument in STATS");
		return 1;
	}

	return 0;
#else
	sendto_dcc(conn->fd, ".stats Disabled.");
	return 0;
#endif
}

static int d_htmlupdate(struct Connection *conn, int parc, char *parv[])
{
#ifdef STATSERV
	if (conn->myuser == NULL)
	{
		log("Access Denied (.htmlupdate) to %s", conn->name);
		spy(".htmlupdate Attempt by %s (failed)", conn->name);
		sendto_dcc_flag(CF_COMMANDS,
			".htmlupdate Attempt by %s (failed)", conn->name);
		sendto_dcc(conn->fd, "Access Denied.");
		/* Not a code error */
		return 0;
	}

	if (!IsHtml())
	{
		sendto_dcc(conn->fd, "HTML Update Disabled");
		return 0;
	}

	DumpHTML();
	spy(".htmlupdate Request by %s", conn->name);
	sendto_dcc_flag(CF_COMMANDS, ".htmlupdate Request by %s", conn->name);
	sendto_dcc(conn->fd, "HTML Update Forced");
	log("%s@fd:%d forces an HTML Update", conn->name, conn->fd);
	return 0;
#else
	sendto_dcc(conn->fd, ".update Disabled.");
	return 0;
#endif
}

static int d_spforget(struct Connection *conn, int parc, char *parv[])
{
#ifdef SPLITSERV
	struct SpServer * s = NULL;
	dlink_node *dl = NULL;

	if (conn->myuser == NULL)
	{
                log("Access Denied (.spforget) to %s", conn->name);
                spy(".spforget Attempt by %s (failed)", conn->name);
		sendto_dcc_flag(CF_COMMANDS, ".spforget Attempt by %s (failed)",
			conn->name);
                sendto_dcc(conn->fd, "Access Denied.");
                /* Not a code error */
                return 0;
	}

        if (parc < 2)
        {
                sendto_dcc(conn->fd, "Syntax: .spforget <server>");
                strcpy(errstr, "Missing parameter in .spforget");
                return 1;
        }

	for (dl = split_table.head; dl; dl = dl->next)
	{
		s = (struct SpServer *) dl->data;
		if (!strcasecmp(s->name, parv[1]))
		{
			sendto_dcc(conn->fd, "Removed from the Split Table",
				s->name);
			dlink_delete(dl, &split_table);
			privmsg(sp.join_chan, SplitServ.nick, MSG_SPDEFAULT,
				"FORGOT SPLIT: %s", s->name);
			splog("Forgot split %s by %s", s->name, conn->name);
			free(s);
			free(dl);
			return 0;
		}
	}

	sendto_dcc(conn->fd, "%s NOT found in the Split Table", parv[1]);
#else
	sendto_dcc(conn->fd, ".spforget Disabled.");
#endif
	return 0;
}

static int d_spadd(struct Connection *conn, int parc, char *parv[])
{
#ifdef SPLITSERV
	struct SpServer *s = NULL;

	if (conn->myuser == NULL)
	{
                log("Access Denied (.spadd) to %s", conn->name);
                spy(".spadd Attempt by %s (failed)", conn->name);
		sendto_dcc_flag(CF_COMMANDS, ".spadd Attempt by %s (failed)",
			conn->name);
                sendto_dcc(conn->fd, "Access Denied.");
                /* Not a code error */
                return 0;
	}

        if (parc < 2)
        {
                sendto_dcc(conn->fd, "Syntax: .spadd <server>");
                strcpy(errstr, "Missing parameter in .spadd");
                return 1;
        }

	s = SpFindServer(parv[1]);
	if (s != NULL)
	{
		sendto_dcc(conn->fd,
			"Can't add a split for an existing server");
		return 0;
	}
	s = SpFindSplit(parv[1]);
	if (s != NULL)
	{
		sendto_dcc(conn->fd, "Can't add a split for an existing split");
		return 0;
	}
	SpNewSplit(parv[1]);
	privmsg(sp.join_chan, SplitServ.nick, MSG_SPDEFAULT,
		"ADDED SPLIT: %s", parv[1]);
	splog("Added split %s by %s", parv[1], conn->name);

#else
	sendto_dcc(conn->fd, ".spadd Disabled.");
#endif
	return 0;
}

static int d_mapupdate(struct Connection *conn, int parc, char *parv[])
{
#ifdef SPLITSERV
	if (conn->myuser == NULL)
	{
                log("Access Denied (.mapupdate) to %s", conn->name);
                spy(".mapupdate Attempt by %s (failed)", conn->name);
		sendto_dcc_flag(CF_COMMANDS,
			".mapupdate Attempt by %s (failed)", conn->name);
                sendto_dcc(conn->fd, "Access Denied.");
                /* Not a code error */
                return 0;
	}

	if (IsSPEnableText())
	{
		dump_map();
		sendto_dcc(conn->fd, "MAP file updated");
		return 0;
	}
	sendto_dcc(conn->fd, "MAP Text Dump Disabled");
#else
	sendto_dcc(conn->fd, ".mapupdate Disabled.");
#endif
	return 0;
}

static int d_kick(struct Connection *conn, int parc, char *parv[])
{
	struct Connection *target;
	int tfd = 0;

        if (conn->myuser == NULL)
        {
                log("Access Denied (.kick) to %s", conn->name);
                spy(".kick Attempt by %s (failed)", conn->name);
                sendto_dcc_flag(CF_COMMANDS,
                        ".kick Attempt by %s (failed)", conn->name);
                sendto_dcc(conn->fd, "Access Denied.");
                /* Not a code error */
                return 0;
        }

	if (parc < 2)
	{
	        sendto_dcc(conn->fd, "Syntax: .kick <fd>");
                strcpy(errstr, "Missing parameter in .kick");
                return 1;
        }

	tfd = atoi(parv[1]);
	if (tfd <= 0)
	{
		sendto_dcc(conn->fd, "Syntax: .kick <fd>");
		strcpy(errstr, "Non-FD parameter in .kick");
		return 1;
	}
	target = find_connection(tfd);
	if (target == NULL)
	{
		sendto_dcc(conn->fd, "No such FD: %s", parv[1]);
		strcpy(errstr, "Invalid FD in .kick");
		return 1;
	}
	if (target->myuser != NULL)
	{
		sendto_dcc(conn->fd, "Cannot kick another logged in user");
		log(".kick %d (%s) Attempt by %s (failed)", target->fd,
			target->name, conn->name);
		spy(".kick %d (%s) Attempt by %s (failed)", target->fd,
			target->name, conn->name);
		sendto_dcc_flag(CF_COMMANDS,
			".kick %d (%s) Attempt by %s (failed)", target->fd,
			target->name, conn->name);
		/* Not a code error */
		return 0;
	}
	/* FINALLY we can remove the client. */
	sendto_dcc(target->fd, "You have been kicked by %s", conn->name);
	sendto_dcc(conn->fd, "Kicking %s", target->name);
	close(target->fd);
	target->flags |= CONN_DEAD;
	sendto_all_dcc("%s has been kicked by %s", target->name, conn->name);
	delete_connection(target->fd);

	return 0;
}

static int d_me(struct Connection *conn, int parc, char *parv[])
{
	char line[BUFSIZE];
	char *ptr;

	ExpandParv(line, BUFSIZE, 1, parc, parv);

	if (line[0] == '\0')
		/* Ignore blank .me's */
		return 0;

	/* If this was caught be someone doing ^AACTION blah blah^A in their
	** client, trim the extra ^A
	*/
	ptr = strchr(line, '\001');
	if (ptr != NULL)
		*ptr = '\0';

	if (conn->flags & CF_CHAT)	
		sendto_partyline(conn, "* %s %s", conn->name, line);
	else
		sendto_dcc(conn->fd, "You are not set +CHAT");

	return 0;
}

static int d_events(struct Connection *conn, int parc, char *parv[])
{
        if (conn->myuser == NULL)
        {
                log("Access Denied (.events) to %s", conn->name);
                spy(".events Attempt by %s (failed)", conn->name);
                sendto_dcc_flag(CF_COMMANDS,
                        ".events Attempt by %s (failed)", conn->name);
                sendto_dcc(conn->fd, "Access Denied.");
                /* Not a code error */
                return 0;
        }
	show_dcc_events(conn->fd);
	return 0;
}
