/* Stats - IRC Statistical Services for Hybrid, csircd, and IRCnet 2.10
** flags.c - Support functions to .flags
**
** Copyright W. Campbell and others.  See README for more details
** Some code Copyright: Jonathan George, Kai Seidler, ircd-hybrid Team,
**                      IRCnet IRCD developers.
**
** $Id: flags.c,v 1.5 2002/10/27 02:58:22 wcampbel Exp $
*/

#include "stats.h"
#include "struct.h"
#include "proto.h"
#include "token.h"
#include "flags.h"

struct Token flags_table[] =
{
        { "AUTH",       CF_AUTH },
        { "NOTICES",    CF_NOTICES },
        { "JUPE",       CF_JUPE },
        { "OPER",       CF_OPER },
        { "COMMANDS",   CF_COMMANDS },
        { "USER",       CF_USER },
        { "KILL",       CF_KILL },
        { "MISC",       CF_MISC },
        { "SERVICES",   CF_SERVICES },
        { "TLD",        CF_TLD },
        { "CHAT",       CF_CHAT },
	{ "ALL",	CF_ALL },
        { NULL,         TOKEN_ERROR }
};

void send_set_flags(struct Connection *conn)
{
	int i;
	char out[LINE];

	out[0] = '\0';

	for (i = 0; flags_table[i].text != NULL; i++)
	{
		/* Don't display ALL ever */
		if (flags_table[i].value == CF_ALL)
			continue;
		if (conn->flags & flags_table[i].value)
		{
			strlcat(out, " ", LINE);
			strlcat(out, flags_table[i].text, LINE);
		}
	}
	if (out[0] != '\0')
	{
		/* Purposefully leave the extra space */
		sendto_dcc(conn->fd, "Set flags:%s", out);
	}
}

void send_unset_flags(struct Connection *conn)
{
	int i;
	char out[LINE];

	out[0] = '\0';

	for (i = 0; flags_table[i].text != NULL; i++)
	{
                /* Don't display ALL ever */
                if (flags_table[i].value == CF_ALL) 
                        continue;
		if (!(conn->flags & flags_table[i].value))
		{
			strlcat(out, " ", LINE);
			strlcat(out, flags_table[i].text, LINE);
		}
	}
	if (out[0] != '\0')
	{
		/* Purposefully leave the extra space */
		sendto_dcc(conn->fd, "Unset flags:%s", out);
	}
}
