#!/usr/bin/awk -f
#
# Stats - IRC Statistical Services for Hybrid, csircd, and IRCnet 2.10
# genhtml.awk - HTML dump generator
#
# Copyright W. Campbell and others.  See README for more details
# Some code Copyright: Jonathan George, Kai Seidler, ircd-hybrid Team,
#                      IRCnet IRCD developers.
#
# $Id: genhtml.awk,v 1.58 2002/10/29 03:17:02 wcampbel Exp $

BEGIN {
  print "/* html.c - generated source file to perform HTML dumps of"
  print "**          the Stats data."
  print "** DO NOT EDIT!!!"
  print "*/"
  print " "
  print "#include \"stats.h\""
  print "#include \"struct.h\""
  print "#include \"proto.h\""
  print " "
  print "void DumpHTML()"
  print "{"
  print "  FILE *fp = fopen(settings.html, \"w\");"
  print "  time_t now = time(NULL);"
  print "  struct Server *s;"
  print "  struct User *u;"
  print "  int i;"
  print "  struct TLD *t;"
  print " "
  print "  int showadmin = 0;"
  print "  int hideuplink = 0;"
  print "  int hideservers = 1;"
  print "  int showinterval = 0;"
  print "#ifdef TRACK_VERSION"
  print "  int hideversion = 0;"
  print "#endif"
  print " "
  print "  if (!fp)"
  print "  {"
  print "    log(\"Unable to open HTML Dump file %s for writing.\","
  print "         settings.html);"
  print "    return;"
  print "  }"
  print " "
  print "  log(\"HTML Dump file %s opened\", settings.html);"
  print " "
  print " /* write out the HTML heading lines */"
  print " "
}

{
  if ($1 == "!NETWORK_DATA!") {
    print " "
    print "#ifdef STATSERV"
    print "  s = findserver(settings.name);"
    print "  if (IsSHide(s) && hideservers)"
    print "    fprintf(fp, \"<B>My Name:</B> %s<BR>\\n\", StatServ.nick);"
    print "  else"
    print "    fprintf(fp, \"<B>My Name:</B> %s!%s@%s on %s<BR>\\n\", StatServ.nick, StatServ.user, StatServ.host, settings.name);"
    print "#endif"
    print "    if (me.s && !(IsSHide(me.s) && hideservers))"
    print "      fprintf(fp, \"<B>My Uplink:</B> %s<BR>\\n\", (me.s) ? me.s->name : \"NONE\");"
    print "  fprintf(fp, \"<B>My Admin:</B> %s<BR>\\n\", settings.admin_name);"
    print "  fprintf(fp, \"<B>Current User Count:</B> %ld<BR>\\n\", netinfo.users);"
    print "  fprintf(fp, \"<B>Max Users:</B> %ld at %s<BR>\\n\", netinfo.maxusers, sftime(netinfo.t_maxusers));"
    print "  /* hybrid doesn't propegate AWAY, so don't print it out */"
    print "#if defined(IRCD_2_10) || defined(IRCD_HYB7) || defined(IRCD_BAHAMUT) || defined(IRCD_HYBSVS)"
    print "  fprintf(fp, \"<B>Current Away User Count:</B> %d (%2.0f%%)<BR>\\n\", netinfo.away, ( (float) netinfo.away / (float) netinfo.users * 100) );"
    print "  fprintf(fp, \"<B>Max Away:</B> %d at %s<BR>\\n\", netinfo.maxaway, sftime(netinfo.t_maxaway));"
    print "#endif"
    print "  fprintf(fp, \"<B>Current Oper Count:</B> %d (%2.0f%%)<BR>\\n\", netinfo.opers, ( (float) netinfo.opers / (float) netinfo.users * 100) );"
    print "  fprintf(fp, \"<B>Max Opers:</B> %d at %s<BR>\\n\", netinfo.maxopers, sftime(netinfo.t_maxopers));"
    print "  fprintf(fp, \"<B>Current Server Count:</B> %d<BR>\\n\", netinfo.servers);"
    print "  fprintf(fp, \"<B>Max Servers:</B> %d at %s<BR>\\n\", netinfo.maxservers, sftime(netinfo.t_maxservers));"
    print "#ifdef IRCD_2_10"
    print "  fprintf(fp, \"<B>Max Services:</B> %d at %s<BR>\\n\", netinfo.maxservices, sftime(netinfo.t_maxservices));"
    print "#endif"
    print "  fprintf(fp, \"<B>Statistics Last Updated:</B> %s<BR>\\n\", sftime(now));"
    print "  fprintf(fp, \"<B>Current Uptime:</B> %s<BR>\\n\", timediff(now - me.t_start));"
    print "  if (showinterval)"
    print "    fprintf(fp, \"<B>HTML Dump Interval:</B> %s<BR>\\n\", timediff(settings.html_time));"
    print " "
  } else if ($1 == "!STATS_VERSION!") {
    print " "
    print "  fprintf(fp, \"<B>Stats Version:</B> %s %s<BR>\\n\", stats_version, VersionFlags());"
    print " "
  } else if ($1 == "!SERVER_LIST!") {
    print " "
    print "  for (i=0; i < S_TABLE_SIZE; i++)"
    print "    for (s = serverlist[i]; s; s = s->next)"
    print "      if (!(IsSHide(s) && hideservers))"
    print "        fprintf(fp, \"<A HREF=\\\"#%s\\\">%s</A><BR>\\n\", s->name, s->name);"
    print " "
  } else if ($1 == "!TABLE_LIST!") {
    print " "
    print "  fprintf(fp, \"<TABLE BORDER=\\\"1\\\"><TR><TH>Server</TH><TH>Users/Max</TH><TH>Opers/Max</TH><TH>Percent</TH></TR>\");"
    print "  for (i=0; i < S_TABLE_SIZE; i++)"
    print "  {"
    print "    for (s = serverlist[i]; s; s = s->next)"
    print "    {"
    print "      if (!(IsSHide(s) && hideservers))"
    print "      {"
    print "          fprintf(fp, \"<TR><TD><A HREF=\\\"#%s\\\">%s</A></TD><TD>%d/%d</TD><TD>%d/%d</TD><TD>%2.0f%%</TD></TR>\\n\","
    print "                  s->name, s->name, s->users, s->stats->maxusers, s->opers, s->stats->maxopers, (float) s->users / (float) netinfo.users * 100);"
    print "      }"
    print "    }"
    print "  }"
    print "  fprintf(fp, \"</TABLE>\\r\");"
    print " "
  } else if ($1 == "!MAP_LIST!") {
    print " "
    print "#ifdef SPLITSERV"
    print "  html_map(fp, map_head, 0, NULL);"
    print "#endif"
    print " "
  } else if ($1 == "!BOT_LIST!") {
    print " "
    print "#ifdef STATSERV"
    print "  fprintf(fp, \"%s!%s@%s<BR>\\n\", StatServ.nick, StatServ.user, StatServ.host);"
    print "#endif"
    print "#ifdef SPLITSERV"
    print "  fprintf(fp, \"%s!%s@%s<BR>\\n\", SplitServ.nick, SplitServ.user, SplitServ.host);"
    print "#endif"
    print "  if (showadmin)"
    print "    fprintf(fp, \"%s!%s@%s<BR>\\n\", AdminServ.nick, AdminServ.user, AdminServ.host);"
    print "#ifdef JUPESERV"
    print "    fprintf(fp, \"%s!%s@%s<BR>\\n\", JupeServ.nick, JupeServ.user, JupeServ.host);"
    print "#endif"
    print " "
  } else if ($1 == "@SHOW_ADMIN@") {
    print " "
    print "  showadmin = 1;"
    print " "
  } else if ($1 == "@HIDE_UPLINK@") {
    print " "
    print "  hideuplink = 1;"
    print " "
  } else if ($1 == "@SHOW_INTERVAL@") {
    print " "
    print "  showinterval = 1;"
    print " "
  } else if ($1 == "@HIDE_VERSION@") {
    print " "
    print "#ifdef TRACK_VERSION"
    print "  hideversion = 1;"
    print "#endif"
    print " "
  } else if ($1 == "@SHOW_HIDDEN@") {
    print " "
    print "  hideservers = 0;"
    print " "
  } else if ($1 == "!TLD_TABLE!") {
    print " "
    print "  for (i = 0; i < T_TABLE_SIZE; i++)"
    print "  {"
    print "    for(t = tldlist[i]; t; t = t->next)"
    print "    {"
    print "      if (t->users > 0)"
    print "      {"
    print "        fprintf(fp, \"%d user%s on %s (%s) (%2.0f%%)<BR>\\n\", t->users,"
    print "        (t->users == 1) ? \"\" : \"s\", t->country, t->tld,"
    print "        (float)t->users / (float)netinfo.users * 100);"
    print "      }"
    print "    }"
    print "  }"
    print " "
  } else if ($1 == "!OPER_LIST!") {
    print " "
    print "  {"
    print "    dlink_node *dl;"
    print "    fprintf(fp, \"<OL>\\n\");"
    print "    for (dl = global_oper_list.head; dl; dl = dl->next)"
    print "    {"
    print "      u = (struct User *) dl->data;"
    print "      if (!(IsSHide(u->server) && hideservers) && IsOper(u))"
    print "      {"
    print "#ifdef OPER_MODES"
    print "        fprintf(fp, \"<LI>[%c] %s on %s<BR>\\n\", opermode(u), u->nick, u->server->name);"
    print "#else"
    print "        fprintf(fp, \"<LI>%s on %s<BR>\\n\", u->nick, u->server->name);"
    print "#endif"
    print "      }"
    print "    }"
    print "    fprintf(fp, \"</OL>\\n\");"
    print "  }"
    print " "
  } else if ($1 == "!SPLIT_LIST!") {
    print " "
    print "#ifdef SPLITSERV"
    print "  {"
    print "    int i;"
    print "    int count = 0;"
    print "    char * td;"
    print "    dlink_node *dl;"
    print "    struct SpServer *spl;"
    print " "
    print "    for (dl = split_table.head; dl; dl = dl->next)"
    print "    {"
    print "      spl = (struct SpServer *) dl->data;"
    print "      td = timediff(now - spl->ts);"
    print "      count++;"
    print "      fprintf(fp, \"%s for %s<BR>\\n\", spl->name, td);"
    print "    }"
    print "    if (!count)"
    print "    {"
    print "      fprintf(fp, \"No Splits\\n\");"
    print "    }"
    print "  }"
    print "#endif"
    print " "
  } else if ($1 == "!VERSION_MAP!") {
    print " "
    print "#ifdef TRACK_VERSION"
    print "  fprintf(fp, \"<TABLE><TR><TH>Server</TH><TH>Version</TH></TR>\\r\");"
    print "  for (i=0; i < S_TABLE_SIZE; i++)"
    print "  {"
    print "    for (s = serverlist[i]; s; s = s->next)"
    print "    {"
    print "      if (!(IsSHide(s) && hideservers))"
    print "      {"
    print "          fprintf(fp, \"<TR><TD>%s</TD><TD>%s</TD></TR>\\r\", s->name, (s->version && s->version[0]) ?  s->version : \"Unknown Version\");"
    print "      }"
    print "    }"
    print "  }"
    print "  fprintf(fp, \"</TABLE>\\r\");"
    print "#endif"
    print " "
  } else if ($1 == "!SERVER_DATA!") {
    print " "
    print "  for (i=0; i < S_TABLE_SIZE; i++)"
    print "    for (s = serverlist[i]; s; s = s->next)"
    print "    {"
    print "      if (!(IsSHide(s) && hideservers))"
    print "      {"
    print "        fprintf(fp, \"<HR>\\n\");"
    print "        fprintf(fp, \"<A NAME=\\\"%s\\\"></A>\", s->name);"
    print "        fprintf(fp, \"<CENTER><H3>%s</H3></CENTER><P>\\n\", s->name);"
    print "        fprintf(fp, \"<B>Description Line:</B> %s<BR>\\n\", s->desc);"
    print "#ifdef TRACK_VERSION"
    print "        if (!hideversion && s->version && s->version[0])"
    print "          fprintf(fp, \"<B>Version:</B> %s<BR>\\n\", s->version);"
    print "#endif"
    print "#ifdef TRACK_ADMIN"
    print "        if (s->admin1[0] != '\\0')"
    print "           fprintf(fp, \"<B>Admin:</B> %s<BR>\\n\", s->admin1);"
    print "        if (s->admin2[0] != '\\0')"
    print "           fprintf(fp, \"<B>Admin:</B> %s<BR>\\n\", s->admin2);"
    print "        if (s->admin3[0] != '\\0')"
    print "           fprintf(fp, \"<B>Admin:</B> %s<BR>\\n\", s->admin3);"
    print "#endif"
    print "        fprintf(fp, \"<B>Current User Count:</B> %d<BR>\\n\", s->users);"
    print "        fprintf(fp, \"<B>Max Users:</B> %ld at %s<BR>\\n\", s->stats->maxusers, sftime(s->stats->t_maxusers));"
    print "        fprintf(fp, \"<B>Current Oper Count:</B> %d<BR>\\n\", s->opers);"
    print "        fprintf(fp, \"<B>Max Opers:</B> %d at %s<BR>\\n\", s->stats->maxopers, sftime(s->stats->t_maxopers));"
    print "#ifdef IRCD_2_10"
    print "        fprintf(fp, \"<B>Current Services Count:</B> %d<BR>\\n\", s->services);"
    print "        fprintf(fp, \"<B>Max Services:</B> %d at %s<BR>\\n\", s->stats->maxservices, sftime(s->stats->t_maxservices));"
    print "#endif"
    print "#if defined(IRCD_HYB7) || defined(IRCD_2_10) || defined(IRCD_BAHAMUT) || defined(HYBSVS)"
    print "        fprintf(fp, \"<B>Current Away Count:</B> %d<BR>\\n\", s->away);"
    print "        fprintf(fp, \"<B>Max Away:</B> %d at %s<BR>\\n\", s->stats->maxaway, sftime(s->stats->t_maxaway));"
    print "#endif"
    print "        fprintf(fp, \"<B>Connected Since:</B> %s<BR>\\n\", sftime(s->connected_since));"
    print "        if (!hideuplink)"
    print "        {"
    print "          if (s->uplink)"
    print "          {"
    print "            if (!(hideservers && (s->uplink->flags & SFLAG_HIDE)))"
    print "              fprintf(fp, \"<B>Uplink:</B> <A HREF=\\\"#%s\\\">%s</A><BR>\\n\", s->uplink->name, s->uplink->name);"
    print "          }"
    print "        }"
    print "    }"
    print "  }"
    print " "
  } else if ($1 == "!DYN_HEADER!") {
    print " "
    print "  fprintf(fp, \"<TITLE>IRC Statistics for %s </TITLE>\\n\", sftime(now));"
    print "  fprintf(fp, \"<META HTTP-EQUIV=\\\"Refresh\\\" CONTENT=\\\"%d\\\">\\n\", settings.html_time);"
    print " "
  } else if ($1 == "!SERVICE_LIST!") {
    print " "
    print "#ifdef IRCD_2_10"
    print "  {"
    print "    dlink_node *dl;"
    print "    struct Service *sv;"
    print "    for (dl = servicehead.head; dl; dl = dl->next)"
    print "    {"
    print "      sv = (struct Service *) dl->data;"
    print "      fprintf(fp, \"%s@%s distribution %s (%s)<BR>\\n\", sv->nick, sv->server, sv->dist, sv->gecos);"
    print "    }"
    print "  }"
    print "#endif"
    print " "
  } else {
    print "     fprintf(fp, \""$0"\\n\");"
  }
}

END {
  print "  fclose(fp);"
  print "}"
}
