/* Stats - IRC Statistical Services for Hybrid, csircd, and IRCnet 2.10
** init.c - Stats initialization functions
**
** Copyright W. Campbell and others.  See README for more details
** Some code Copyright: Jonathan George, Kai Seidler, ircd-hybrid Team,
**                      IRCnet IRCD developers.
**
** $Id: init.c,v 1.20 2002/10/29 19:27:01 wcampbel Exp $
*/

#include "stats.h"
#include "struct.h"
#include "proto.h"

#ifdef SUPPORT_MSG_UH
#ifdef STATSERV
char s_UH[MAXUSER + MAXHOST + 2];
#endif
#ifdef SPLITSERV
char sp_UH[MAXUSER + MAXHOST + 2];
#endif
#ifdef JUPESERV
char j_UH[MAXUSER + MAXHOST + 2];
#endif
char a_UH[MAXUSER + MAXHOST + 2];
#endif

#ifdef SUPPORT_MSG_OPERS
/* 7 is "opers@" and the \0 terminator
** I should PROBABLY force this to only be enabled on Hybrid 6
*/
char s_OPERS[MAXHOST + 7];
#endif

/* These are all horrible functions, peppered with ifdef's */

/* This is ugly, logging into each server type is slightly different */
void login()
{
#if defined(IRCD_HYBRID)
	sts("PASS %s :TS", settings.pass);
	sts("SERVER %s 1 :%s", settings.name, settings.desc);
	sts("SVINFO 3 1 0 :%ld", time(NULL));
#elif defined(IRCD_HYB6) || defined(IRCD_HYB7) || defined(IRCD_HYBSVS)
	sts("PASS %s :TS", settings.pass);
	sts("CAPAB :%s", me.capabs);
        sts("SERVER %s 1 :%s", settings.name, settings.desc);
        sts("SVINFO 5 3 0 :%ld", time(NULL));
#elif defined(IRCD_2_10)
	sts("PASS %s :021030003 IRC|%sHst P", settings.pass,
		IsDebug() ? "D" : "");
	sts("SERVER %s 1 :%s", settings.name, settings.desc);
#elif defined(IRCD_BAHAMUT)
	sts("PASS %s :TS", settings.pass);
	/* Using SSJOIN will save bandwidth, UNCONNECT is required */
	sts("CAPAB :%s", me.capabs);
	sts("SERVER %s 1 :%s", settings.name, settings.desc);
	sts("SVINFO 3 1 0 :%ld", time(NULL));
#endif
}

#ifdef JUPESERV

/* Bring up the jupe service */
void init_jupeserv()
{
        struct User *u = NULL;

#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7) || \
    defined(IRCD_HYBSVS)
        sts("NICK %s 1 1 +io %s %s %s :Jupe Service",
                JupeServ.nick, JupeServ.user, JupeServ.host, settings.name);
#elif defined(IRCD_2_10)
        sts("NICK %s 1 %s %s %d +io :Jupe Service", JupeServ.nick,
               JupeServ.user, JupeServ.host, 1);
#elif defined(IRCD_BAHAMUT)
	sts("NICK %s 1 1 +io %s %s %s 1 :Jupe Service",
		JupeServ.nick, JupeServ.user, JupeServ.host, settings.name);
#endif

#ifdef IRCD_2_10
        AddUser(JupeServ.nick, JupeServ.user, JupeServ.host,
                findserver(settings.name));
#else
        AddUser(JupeServ.nick, JupeServ.user, JupeServ.host, settings.name);
#endif

        u = finduser(JupeServ.nick);
        IncreaseOpers(findserver(settings.name));
        u->flags |= UFLAG_ISOPER;
#ifdef OPER_MODES
        u->operflags |= OPER_MYOPER;
#endif
	AddOperToList(&global_oper_list, u);
}

#endif

#ifdef STATSERV
/* Init the StatServ client */
void init_statserv()
{

#ifdef SUPPORT_MSG_UH
	snprintf(StatServ.rname, LINE, "/msg %s HELP", s_UH);
#else
	snprintf(StatServ.rname, LINE, "/msg %s HELP", StatServ.nick);
#endif

#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7) || \
    defined(IRCD_HYBSVS)
	sts("NICK %s 1 1 %s %s %s %s :%s", StatServ.nick,
		IsSetOper() ? "+o" : "+", StatServ.user, StatServ.host,
		settings.name, StatServ.rname);
#elif defined(IRCD_2_10)
	sts("NICK %s 1 %s %s %d %s :%s", StatServ.nick,
		StatServ.user, StatServ.host, 
		/* I am Server ID 1 */
		1, IsSetOper() ? "+o" : "+", StatServ.rname);
#elif defined(IRCD_BAHAMUT)
	sts("NICK %s 1 1 %s %s %s %s 1 :%s", StatServ.nick,
		IsSetOper() ? "+o" : "+", StatServ.user, StatServ.host,
		settings.name, StatServ.rname);
#endif

#ifdef IRCD_2_10
	AddUser(StatServ.nick, StatServ.user, StatServ.host,
		findserver(settings.name));
#else
	AddUser(StatServ.nick, StatServ.user, StatServ.host, settings.name);
#endif

/* If the StatServ client is an oper, increase oper count here
** and set it as an oper
*/
	if (IsSetOper()) {
		struct User* u = finduser(StatServ.nick);
		IncreaseOpers(findserver(settings.name));
		u->flags |= UFLAG_ISOPER;
#ifdef OPER_MODES
		u->operflags |= OPER_MYOPER;
#endif
		AddOperToList(&global_oper_list, u);
	}
}
#endif /* STATSERV */

#ifdef SPLITSERV
/* Bring up the Split detecting/reporting client if it was enabled */
void init_splitserv()
{

#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7) || \
    defined(IRCD_HYBSVS)
        sts("NICK %s 1 1 + %s %s %s :%s", SplitServ.nick, SplitServ.user,
                SplitServ.host, settings.name, sp.desc);
#elif defined(IRCD_2_10)
        sts("NICK %s 1 %s %s %d + :%s", SplitServ.nick, SplitServ.user,
                SplitServ.host, 1, sp.desc);
                /* This is the server token, myself is 1 */
#elif defined(IRCD_BAHAMUT)
	sts("NICK %s 1 1 + %s %s %s 1 :%s", SplitServ.nick, SplitServ.user,
		SplitServ.host, settings.name, sp.desc);
#endif

#ifdef IRCD_2_10
        AddUser(SplitServ.nick, SplitServ.user, SplitServ.host,
		findserver(settings.name));
#else
        AddUser(SplitServ.nick, SplitServ.user, SplitServ.host, settings.name);
#endif
}
#endif

/* Bring up the administration service */
void init_admin()
{
	struct User *u = NULL;

#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7) || \
    defined(IRCD_HYBSVS)
        sts("NICK %s 1 1 +io %s %s %s :Administration Service",
		AdminServ.nick, AdminServ.user, AdminServ.host, settings.name);
#elif defined(IRCD_2_10)
        sts("NICK %s 1 %s %s %d +io :Administration Service", AdminServ.nick,
		AdminServ.user, AdminServ.host, 1);
#elif defined(IRCD_BAHAMUT)
	sts("NICK %s 1 1 +io %s %s %s 1 :Administration Service",
		AdminServ.nick, AdminServ.user, AdminServ.host, settings.name);
#endif

#ifdef IRCD_2_10
        AddUser(AdminServ.nick, AdminServ.user, AdminServ.host,
		findserver(settings.name));
#else
        AddUser(AdminServ.nick, AdminServ.user, AdminServ.host, settings.name);
#endif

	u = finduser(AdminServ.nick);
	IncreaseOpers(findserver(settings.name));
	u->flags |= UFLAG_ISOPER;
#ifdef OPER_MODES
        u->operflags |= OPER_MYOPER;
#endif
	AddOperToList(&global_oper_list, u);
}

#ifdef STATSERV
/* Join the channel and obtain ops if it is a new channel
** We don't take over channels with Stats
*/
void init_join()
{
   dlink_node *dl = me.chan_list.head;

   while (dl)
   {
#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7) || \
    defined(IRCD_BAHAMUT) || defined(IRCD_HYBSVS)
	sts(":%s SJOIN %ld %s + :@%s", settings.name, 
	    time(NULL), (char *) dl->data, StatServ.nick);
#elif defined(IRCD_2_10)
	/* Here is the only one that we hack ops... */
	sts(":%s NJOIN %s :@%s", settings.name, (char *) dl->data, 
		StatServ.nick);
#endif
	dl = dl->next;
   }
}
#endif

#ifdef SPLITSERV
/* Join the channel and obtain ops if it is a new channel
** We don't take over channels with Split
*/
void init_spjoin()
{
#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7) || \
    defined(IRCD_BAHAMUT) || defined(IRCD_HYBSVS)
	sts(":%s SJOIN %ld %s + :@%s", settings.name,
        	time(NULL), sp.join_chan, SplitServ.nick);
#elif defined(IRCD_2_10)
        /* Here is the only one that we hack ops... */
        sts(":%s NJOIN %s :@%s", settings.name, sp.join_chan, SplitServ.nick);
#endif
}
#endif

#ifdef ALERTMODE
/* Join the channel and obtain ops if it is a new channel
** We don't take over channels with AdminServ...
** Though in this case we might want to.....
*/
void init_alert_join()
{
#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7) || \
    defined(IRCD_BAHAMUT) || defined(IRCD_HYBSVS)
        sts(":%s SJOIN %ld %s + :@%s", settings.name,
                time(NULL), ALERTCHAN, AdminServ.nick);
#elif defined(IRCD_2_10)
        /* Here is the only one that we hack ops... */
        sts(":%s NJOIN %s :@%s", settings.name, ALERTCHAN, AdminServ.nick);
#endif
}
#endif

#ifdef SPYMODE
/* Join the channel and obtain ops if it is a new channel
** We don't take over channels with AdminServ...
** Though in this case we might want to.....
*/
void init_spy_join()
{
#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7) || \
    defined(IRCD_BAHAMUT) || defined(IRCD_HYBSVS)
        sts(":%s SJOIN %ld %s + :@%s", settings.name,
                time(NULL), SPYCHAN, AdminServ.nick);
#elif defined(IRCD_2_10)
        /* Here is the only one that we hack ops... */
        sts(":%s NJOIN %s :@%s", settings.name, SPYCHAN, AdminServ.nick);
#endif
}
#endif

void InitMessageStrings()
{
#ifdef SUPPORT_MSG_UH
        /* Create the user@host that we are to save some cycles later...
        ** IF this features is used often
        */
#ifdef STATSERV
        snprintf(s_UH, (MAXHOST + MAXUSER + 2), "%s@%s", StatServ.user,
                settings.name);
#endif
#ifdef SPLITSERV
        snprintf(sp_UH, (MAXHOST + MAXUSER + 2), "%s@%s", SplitServ.user,
                settings.name);
#endif
        snprintf(a_UH, (MAXHOST + MAXUSER + 2), "%s@%s", AdminServ.user,
                settings.name);
#ifdef JUPESERV
        snprintf(j_UH, (MAXHOST + MAXUSER + 2), "%s@%s", JupeServ.user,
                settings.name);
#endif
#endif

#ifdef SUPPORT_MSG_OPERS
        /* Same as above...If we call it once, we'll definately call it
        ** again...
        */
        snprintf(s_OPERS, (MAXHOST + 7), "opers@%s", settings.name);
#endif
}
