/* Stats - IRC Statistical Services for Hybrid, csircd, and IRCnet 2.10
** jupe.c - JupeServ backend functions
**
** Copyright W. Campbell and others.  See README for more details
** Some code Copyright: Jonathan George, Kai Seidler, ircd-hybrid Team,
**                      IRCnet IRCD developers.
**
** $Id: jupe.c,v 1.31 2002/09/29 03:03:06 wcampbel Exp $
*/

#include "stats.h"
#include "struct.h"
#include "proto.h"

#ifdef JUPESERV

static void LoadOldJupes();

void LoadJupes()
{
        FILE *fp;
        int parc;
        int i;
        char *parv[MAXPARA];
        char buf[BUFSIZE];

        /* Clear the parv */
        for (i = 0; i < MAXPARA; i++)
                parv[i] = NULL;

	/* New format jupe configuration file */
        if ((fp = fopen(JUPEFILE, "r")) == NULL)
        {
                log("Unable to load the jupe configuration file.");

		/* Check to see if there's an old style conf */
		LoadOldJupes();
                return;
        }

        while (fgets(buf, BUFSIZE, fp))
        {
                parc = tabtoken(buf, parv);

                if (parc < 4)
                {
                        log("Invalid entry in jupe configuration file");
                        log("Ignoring...please correct");
                        continue;
                }

                AddJupe(parv[0], parv[1], atol(parv[2]), parv[3]);
        }
        fclose(fp);
}

void LoadOldJupes()
{
        FILE *fp;
        int parc;
        int i;
        char *parv[MAXPARA];
        char buf[BUFSIZE];

        /* Clear the parv */
        for (i = 0; i < MAXPARA; i++)
                parv[i] = NULL;

        if ((fp = fopen(OLDJUPEFILE, "r")) == NULL)
        {
                log("Unable to load the old style jupe configuration file.");
                return;
        }

        while (fgets(buf, BUFSIZE, fp))
        {
                parc = msgtoken(buf, parv);

                if (parc < 3)
                {
                        log("Invalid entry in jupe configuration file");
                        log("Ignoring...please correct");
                        continue;
                }

                AddJupe(parv[0], parv[1], atol(parv[2]), "No Reason");
        }
        fclose(fp);

	/* We don't want this old style conf around anymore */
	SaveJupes();
	unlink(OLDJUPEFILE);
}

void SaveJupes()
{
        struct Jupe *j;
	dlink_node *dl;
        FILE *fp;

        /* I should zero out the jupe config file here */
        if (jupehead.head == NULL)
	{
		truncate(JUPEFILE, 0);
                return;
	}

        if ((fp = fopen(JUPEFILE, "w")) == NULL)
        {
                log("Unable to open jupe configuration file for writing");
                return;
        }

        for (dl = jupehead.head; dl; dl = dl->next)
        {
		j = (struct Jupe *) dl->data;
                fprintf(fp, "%s\t%s\t%ld\t%s\n", j->jupe, j->oper, j->ts,
			j->reason);
        }
        fclose(fp);
}

void AddJupe(char *jupe, char *oper, long ts, char *reason)
{
        struct Jupe *j = smalloc(sizeof(struct Jupe));
        struct Jupe *check = FindJupe(jupe);
	dlink_node *dl = smalloc(sizeof(dlink_node));

        /* Don't add a jupe if it's already there */
        if (check != NULL)
	{
		/* but only after you free what you allocated... */
		free(j);
		free(dl);
                return;
	}

        strlcpy(j->jupe, jupe, MAXHOST);
        strlcpy(j->oper, oper, MAXNICK + 1);
        j->ts = ts;
	strlcpy(j->reason, reason, LINE);

	dlink_add(j, dl, &jupehead);

        /* Added the data structure Jupe record, now force it off the
        ** network and introduce our fake server in it's place
        */
	InitJupe(j);
}

void InitJupe(struct Jupe *j)
{
	struct Server *s = findserver(j->jupe);
        char jreason[LINE];
#ifdef SPLITSERV
        time_t now = time(NULL);
        struct SpServer *spl = NULL;
        char *td = NULL;
#endif

        if (s)
                sts(":%s SQUIT %s :JUPED by %s - %s", JupeServ.nick, j->jupe,
			j->oper, j->reason);

	snprintf(jreason, LINE, "{Jupitered} by %s - %s", j->oper, j->reason);
        sts(":%s SERVER %s 2 :%s", settings.name, j->jupe,
		jreason);

	AddServer(j->jupe, 1, jreason, settings.name);
#ifdef SPLITSERV
	spl = SpFindSplit(j->jupe);
	if (spl)
	{
		td = timediff(now - spl->ts);
		splog("End of Split: %s to %s: %s", j->jupe, settings.name,
			td);
		if (IsSPReport())
			privmsg(sp.join_chan, SplitServ.nick, MSG_SPDEFAULT,
				"END OF SPLIT: %s to %s: %s", j->jupe,
				settings.name, td);
	}
	else
	{
		splog("New Server: %s to %s", j->jupe, settings.name);
		if (!is_delayed)
		{
			if (IsSPReport())
				privmsg(sp.join_chan, SplitServ.nick,
				MSG_SPDEFAULT, "CONNECT: %s to %s", j->jupe,
				settings.name);
		}
	}
	SpNewServer(j->jupe, settings.name);
	SpDelSplit(j->jupe);
#endif
}

void DelJupe(char *jupe)
{
        struct Jupe *j;
	dlink_node *dl = jupehead.head;

        while (dl)
        {
		j = (struct Jupe *) dl->data;
                if (!strcasecmp(j->jupe, jupe))
                {
			dlink_delete(dl, &jupehead);
                        free(j);
			free(dl);

                        /* Remove the juped server from the network */
                        sts("SQUIT %s :Unjuped", jupe);

			DelServer(jupe);

                        return;
                }
                else
                {
                        dl = dl->next;
                }
        }

        log("Unable to delete jupe %s", jupe);
}

struct Jupe* FindJupe(char *jupe)
{
        struct Jupe *j;
	dlink_node *dl = jupehead.head;

        if (IsDebug())
                log("FindJupe(%s)", jupe);

        while (dl)
        {
		j = (struct Jupe *) dl->data;
                if (!strcasecmp(jupe, j->jupe))
                {
                        return j;
                }
                else
                {
                        dl = dl->next;
                }
        }

        return NULL;
}

void FlushJupeList()
{
	dlink_node *dl;
	struct Jupe *j;

        while (jupehead.head != NULL)
        {
		dl = jupehead.head;
		j = (struct Jupe *) dl->data;
		dlink_delete(dl, &jupehead);
		free(j);
                free(dl);
        }
}

#endif
