/* Stats - IRC Statistical Services for Hybrid, csircd, and IRCnet 2.10
** jupeserv.c - Jupe service functionality
**
** Copyright W. Campbell and others.  See README for more details
** Some code Copyright: Jonathan George, Kai Seidler, ircd-hybrid Team,
**                      IRCnet IRCD developers.
**
** $Id: jupeserv.c,v 1.31 2002/10/24 18:54:50 wcampbel Exp $
*/

/* Some modifications made by Pantheon <kllrloop@kllrloop.com> */

#include "stats.h"
#include "struct.h"
#include "proto.h"

#ifdef JUPESERV

static char errstr[BUFSIZE];

static int j_jupe(char *, int, char **);
static int j_unjupe(char *, int, char **);
static int j_help(char *, int, char **);
static int j_cping(char *, int, char **);
static int j_cversion(char *, int, char **);

static struct MsgTable jupe_table[] =
{
        { "HELP",       0,0,NULL,      j_help },
        { "JUPE",       0,0,NULL,      j_jupe },
        { "UNJUPE",     0,0,NULL,      j_unjupe },
        { "\001PING",   0,0,NULL,      j_cping },
        { "\001VERSION\001",    0,0,NULL,      j_cversion },
        { (char *)0,    0,0,NULL,      (int (*)()) 0 }
};

static struct HelpTable help_table[] = {
        { "JUPE",       "help/jupeserv/jupe",           0 },
        { "UNJUPE",     "help/jupeserv/unjupe",         0 },
        { (char *)0,    (char *)0,                      0 }
};


void jupeserv(char *origin, char *param)
{
        char *parv[MAXPARA];
        int parc = 0;
        struct User *u = NULL;
        int i;
        int status;
        static char buf[BUFSIZE];

        /* Clear the parv */
        for (i = 0; i < MAXPARA; i++)
                parv[i] = NULL;

        /* Clear the error string */
        errstr[0] = '\0';

        strlcpy(buf, param, BUFSIZE);

        if (param)
                parc = msgtoken(param, parv);

        if (origin)
                u = finduser(origin);
        else
                return;

        if (!u)
        {
                log("Unable to finduser %s (adminserv)", origin);
                return;
        }

        netinfo.requests++;

        if (!parc)
                return;

        if (parv[0][0] != '\001')
                log("%s received message from %s: %s", JupeServ.nick, 
                        origin, buf);

        if (IsOnlyOpers() && !IsOper(u))
        {
                if (parv[0][0] != '\001')
                {
                        privmsg(u->nick, JupeServ.nick, MSG_DEFAULT,
                                 "This service is only available to IRCops.");
                        return;
                }
        }

        /* All the preliminary junk is done, now we have to go through 
        ** the tables and call the handler for that command
        */
        for (i=0; jupe_table[i].handler; i++)
        {
                if (!strcasecmp(parv[0], jupe_table[i].name))
                {
                    status = jupe_table[i].handler(u->nick, parc, parv);
                    if (status == 1)
                        log("Error with jupeserv command %s: %s", parv[0],
                        (errstr) ? errstr : "No error message");
                    /* Matched a value, done, then return */
                    return;
                }
        }
        /* If we get here, we did not get a valid command...
        ** Make sure not to say anything to the user if it was a 
        ** CTCP that was not matched
        */
        if (*parv[0] == '\001')
                return;
        privmsg(u->nick, JupeServ.nick, MSG_DEFAULT, "Invalid command: %s",
		parv[0]);
        return;
}

/* In all of these functions, parv[0] contains the command, followed
** by the parameters
*/

static int j_help(char *origin, int parc, char *parv[])
{
        struct User *u = finduser(origin);
        int i;

	spy("%s:  HELP Request by %s for %s", JupeServ.nick, origin,
		(parv[1]) ? parv[1] : "NO QUERY");
	sendto_dcc_flag(CF_COMMANDS, "%s:  HELP Request by %s for %s",
		JupeServ.nick, origin, (parv[1]) ? parv[1] : "NO QUERY");
        if (parc < 2)
        {
		send_help_file(u->nick, JupeServ.nick, MSG_DEFAULT,
			"help/jupeserv/index", "index");
		if (u->myuser != NULL)
			send_help_file(u->nick, JupeServ.nick, MSG_DEFAULT,
				"help/jupeserv/aindex", "aindex");
		return 0;
	}

        for (i = 0; help_table[i].filename; i++)
        {
                if (!strcasecmp(parv[1], help_table[i].name))
                {
                        if (help_table[i].myuonly)
                        {
                                if (u->myuser)
					send_help_file(u->nick, JupeServ.nick,
                                                MSG_DEFAULT,
                                                help_table[i].filename,
                                                help_table[i].name);
                                else
                                        privmsg(u->nick, JupeServ.nick,
                                                MSG_DEFAULT,
                                                "Permission Denied.");
                        }
                        else
                        {
				send_help_file(u->nick, JupeServ.nick,
					MSG_DEFAULT,
                                        help_table[i].filename,
                                        help_table[i].name);
                        }

                        return 0;
                }
        }

        privmsg(u->nick, JupeServ.nick, MSG_DEFAULT,
		"Unknown Help Topic: \2%s\2", parv[1]);
        snprintf(errstr, BUFSIZE, "Unknown HELP topic: %s", parv[1]);
        return 1;
}

static int j_jupe(char *origin, int parc, char *parv[])
{
        struct User *u = finduser(origin);
        struct Jupe *j;
        char reason[LINE];

        if (!u->myuser)
        {
                log("Access denied (JUPE) to %s", u->nick);
                privmsg(u->nick, JupeServ.nick, MSG_DEFAULT, "Access Denied.");
		spy("%s: JUPE %s Attempt by %s (failed)", JupeServ.nick,
			(parv[1]) ? parv[1] : "<nothing>", origin);
		sendto_dcc_flag(CF_JUPE,
			"%s: JUPE %s Attempt by %s (failed)", JupeServ.nick,
			(parv[1]) ? parv[1] : "<nothing>", origin);
                return 0;
        }

        if (parc < 2)
        {
                privmsg(u->nick, JupeServ.nick, MSG_DEFAULT,
                        "Syntax:  /msg %s JUPE <server> <reason>",
			JupeServ.nick);
		spy("%s: JUPE Attempt by %s (failed) (no parameter)",
			JupeServ.nick, origin);
		sendto_dcc_flag(CF_JUPE,
			"%s: JUPE Attempt by %s (failed) (no parameter)",
			JupeServ.nick, origin);
                strcpy(errstr, "Missing parameter in JUPE");
                return 1;
        }

        j = FindJupe(parv[1]);

        if (j)
        {
                privmsg(u->nick, JupeServ.nick, MSG_DEFAULT,
                        "Cannot jupe already juped server");
		spy("%s: JUPE %s Attempt by %s (already juped)", JupeServ.nick,
			parv[1], origin);
		sendto_dcc_flag(CF_JUPE,
			"%s: JUPE Attempt by %s (failed) (no parameter)",
			JupeServ.nick, origin);
                /* Not a code error */
                return 0;
        }

	if (strchr(parv[1], '.') == NULL)
	{
		privmsg(u->nick, JupeServ.nick, MSG_DEFAULT,
			"Only servers may be juped");
		spy("%s: JUPE %s Attempt by %s (non-server)", JupeServ.nick,
			parv[1], origin);
		sendto_dcc_flag(CF_JUPE,
			"%s: JUPE %s Attempt by %s (non-server)",
			JupeServ.nick,
			parv[1], origin);
		/* Not a code error */
		return 0;
	}

	if (strcasecmp(me.s->name, parv[1]) == 0)
	{
		privmsg(u->nick, JupeServ.nick, MSG_DEFAULT,
			"Cannot jupe your uplink");
		spy("%s: JUPE %s Attempt by %s (uplink)", JupeServ.nick,
			parv[1], origin);
		sendto_dcc_flag(CF_JUPE,
			"%s: JUPE %s Attempt by %s (uplink)", JupeServ.nick,
			parv[1], origin);
		/* Not a code error */
		return 0;
	}

	if (strcasecmp(settings.name, parv[1]) == 0)
	{
		privmsg(u->nick, JupeServ.nick, MSG_DEFAULT,
			"Cannot jupe myself");
		spy("%s: JUPE %s Attempt by %s (myself)", JupeServ.nick,
				parv[1], origin);
		sendto_dcc_flag(CF_JUPE,
			"%s: JUPE %s Attempt by %s (myself)", JupeServ.nick,
			parv[1], origin);
                /* Not a code error */
                return 0;
        }

        reason[0] = '\0';
	ExpandParv(reason, LINE, 2, parc, parv);
	if (reason[0] == '\0')
	{
		strcpy(reason, "No Reason");
	}

	/* Wallops routine changed to issue nick!identd@host for other IRCops
	** to see in wallops - Pantheon 10-15-02
	*/

        AddJupe(parv[1], u->myuser->username, time(NULL), reason);
	wallops(settings.name, "%s!%s@%s adding JUPE for %s - Reason: %s",
		origin, u->username, u->hostname, parv[1], reason);
	privmsg(u->nick, JupeServ.nick, MSG_DEFAULT, "Jupe %s added, enforced",
		parv[1]);
	alert("%s: JUPE %s by %s: %s", JupeServ.nick, parv[1], origin, reason);
	sendto_dcc_flag(CF_JUPE, "%s: JUPE %s by %s: %s", JupeServ.nick,
		parv[1], origin, reason);
        SaveJupes();

        return 0;
}

static int j_unjupe(char *origin, int parc, char *parv[])
{
        struct User *u = finduser(origin);
        struct Jupe *j;

        if (!u->myuser)
        {
                log("Access denied (UNJUPE) to %s", u->nick);
		spy("%s: UNJUPE %s Attempt by %s (failed)", JupeServ.nick,
			(parv[1]) ? parv[1] : "<nothing>", origin);
		sendto_dcc_flag(CF_JUPE,
			"%s: UNJUPE %s Attempt by %s (failed)", JupeServ.nick,
			(parv[1]) ? parv[1] : "<nothing>", origin);
                privmsg(u->nick, JupeServ.nick, MSG_DEFAULT, "Access Denied.");
                return 0;
        }

        if (parc < 2)
        {
                privmsg(u->nick, JupeServ.nick, MSG_DEFAULT,
                        "Syntax:  /msg %s UNJUPE <server>", JupeServ.nick);
		spy("%s: UNJUPE Attempt by %s (failed) (no parameter)",
			JupeServ.nick, origin);
		sendto_dcc_flag(CF_JUPE,
			"%s: UNJUPE Attempt by %s (failed) (no parameter)",
			JupeServ.nick, origin);
                strcpy(errstr, "Missing parameter in UNJUPE");
                return 1;
        }

        j = FindJupe(parv[1]);

        if (!j)
        {
                privmsg(u->nick, JupeServ.nick, MSG_DEFAULT,
                        "Jupe %s does not exist", parv[1]);
		spy("%s: UNJUPE %s Attempt by %s (jupe does not exist)",
			JupeServ.nick, parv[1], origin);
		sendto_dcc_flag(CF_JUPE,
			"%s: UNJUPE %s Attempt by %s (jupe does not exist)",
			JupeServ.nick, parv[1], origin);
                /* Not a code error */
                return 0;
        }

        DelJupe(parv[1]);
	wallops(settings.name, "%s!%s@%s removing JUPE for %s", origin,
		u->username, u->hostname, parv[1]);
	alert("%s: UNJUPE %s by %s", JupeServ.nick, parv[1], origin);
	sendto_dcc_flag(CF_JUPE, "%s: UNJUPE %s by %s", JupeServ.nick, parv[1],
		origin);
	privmsg(u->nick, JupeServ.nick, MSG_DEFAULT, "Jupe %s removed",
		parv[1]);
        SaveJupes();

        return 0;
}

static int j_cping(char *origin, int parc, char *parv[])
{
        struct User *u = finduser(origin);
        char command[BUFSIZE];

	ExpandParv(command, BUFSIZE, 1, parc, parv);
        if (command[0] == '\0')
                return 1;
        sts(":%s NOTICE %s :\001PING %s", JupeServ.nick, u->nick, command);
        log("CTCP PING from %s!%s@%s", u->nick, u->username, u->hostname);

        return 0;
}

static int j_cversion(char *origin, int parc, char *parv[])
{
        struct User *u = finduser(origin);

        sts(":%s NOTICE %s :\001VERSION %s %s %s -> %s\001",
             JupeServ.nick, u->nick, stats_version, settings.name,
             VersionFlags(), stats_date);
        log("CTCP VERSION from %s!%s@%s", u->nick, u->username, u->hostname);

        return 0;
}

#endif
