/* Stats - IRC Statistical Services for Hybrid, csircd, and IRCnet 2.10
** myusers.c - Functions to track logged in users
**
** Copyright W. Campbell and others.  See README for more details
** Some code Copyright: Jonathan George, Kai Seidler, ircd-hybrid Team,
**                      IRCnet IRCD developers.
**
** $Id: myusers.c,v 1.4 2002/10/04 15:01:23 wcampbel Exp $
*/

#include "stats.h"
#include "struct.h"
#include "proto.h"

struct MyUser *myuhead;

void LoadMyUsers()
{
	FILE *fp;
	int parc;
	int i;
	char *parv[MAXPARA];
	char buf[BUFSIZE];
	struct MyUser *myu;

        /* Clear the parv */
        for (i = 0; i < MAXPARA; i++)
                parv[i] = NULL;

	if ((fp = fopen(USERFILE, "r")) == NULL)
	{
		fprintf(stderr, "Please type this: echo \"<nickname> "
			"<password> 0 >> %s\"\n", USERFILE);
		fprintf(stderr, "Replace nickname with your nickname, and "
			"password with your password, encrypting if "
			"necessary\n");
		fprintf(stderr, "This is used for the LOGIN feature of "
			"Stats.\n");
		log("Unable to open userlist.");
		exit(0);
	}

	while (fgets(buf, BUFSIZE, fp))
	{
		parc = msgtoken(buf, parv);

		if (parc < 2)
		{
			fprintf(stderr, "Invalid entry in user file: %s\n",
				USERFILE);
			fprintf(stderr, "Format: nick pass 0\n");
			fprintf(stderr, "Or: nick pass <time last seen>\n");
			fprintf(stderr, "Exiting...\n");
			exit(0);
		}

                myu = smalloc(sizeof(struct MyUser));
                strlcpy(myu->username, parv[0], MAXNICK + 1);
                strlcpy(myu->password, parv[1], LINE);

		if (parc < 3)
		{
			log("Invalid time value in user file %s, defaulting "
				"to 0", USERFILE);
			/* smalloc'ed ram */
		}
		else
		{
			myu->lastseen = atol(parv[2]);
		}

		if (!myuhead)
		{
			myuhead = myu;
			myuhead->next = NULL;
		}
		else
		{
			myu->next = myuhead;
			myuhead = myu;
		}
	}
	fclose(fp);
}

void SaveMyUsers()
{
	struct MyUser *myu;
	FILE *fp;

	if (!myuhead)
		return;

	if ((fp = fopen(USERFILE, "w")) == NULL)
	{
		log("Unable to open %s for writing.", USERFILE);
		return;
	}
	for (myu = myuhead; myu; myu = myu->next)
	{
		fprintf(fp, "%s %s %ld\n", myu->username, myu->password,
			myu->lastseen);
	}
	fclose(fp);
}

void DeleteMyUser(char *username)
{
	struct MyUser *myu, *prev = NULL;

	if (!myuhead)
		return;

	myu = myuhead;

	while (myu)
	{
		if (!strcasecmp(myu->username, username))
			break;
		prev = myu;
		myu = myu->next;
	}
	if (!myu)
		return;

	if (!prev)
	{
		myuhead = myu->next;
	}
	else if (myu->next == NULL)
	{
		prev->next = NULL;
	}
	else
	{
		prev->next = myu->next;
	}
	free(myu);
}

struct MyUser *findmyuser(char *nick)
{
	struct MyUser *myu;

	for (myu = myuhead; myu; myu = myu->next)
	{
		if (!strcasecmp(nick, myu->username))
			return myu;
	}
	return NULL;
}
