/* Stats - IRC Statistical Services for Hybrid, csircd, and IRCnet 2.10
** send.c - Functions to send data to sockets
**
** Copyright W. Campbell and others.  See README for more details
** Some code Copyright: Jonathan George, Kai Seidler, ircd-hybrid Team,
**                      IRCnet IRCD developers.
**
** $Id: send.c,v 1.2 2002/10/28 15:40:12 wcampbel Exp $
*/

#include "stats.h"
#include "struct.h"
#include "proto.h"
#include "setup.h"
#include "event.h"

static int sendto_dcc_nr(int, char *, ...);
static int vsendto_dcc(int, char *, va_list);

/* send a line of text to the server, Append the \n */
void sts(char *fmt,...)
{
	va_list ap;
	char buf[BUFSIZE];

	va_start (ap, fmt);
	vsnprintf (buf, BUFSIZE, fmt, ap);

	debug("SENT: %s", buf);
#ifdef IO_LOG
	iolog("S: %s", buf);
#endif
	strlcat(buf, "\n", BUFSIZE);

	netinfo.outb += strlen(buf);

	if ((write(servsock, buf, strlen(buf))) == -1)
	{
		log("Write error.");
		close(servsock);
		delete_connection(servsock);
		servsock = -1;
	}

	va_end (ap);
}

int sendto_dcc(int fd, char *fmt,...)
{
        va_list ap;
        char buf[BUFSIZE];
	struct Connection *conn;

        va_start (ap, fmt);
        vsnprintf (buf, BUFSIZE, fmt, ap);

        debug("DCC SENT: %s", buf);
#ifdef IO_LOG
        iolog("DS: %s", buf);
#endif
        strlcat(buf, "\n", BUFSIZE);

        /* netinfo.outb += strlen(buf); */

        if ((write(fd, buf, strlen(buf))) == -1)
        {
		conn = find_connection(fd);
                log("Write error. Client %s FD: %d", conn->name, fd);
                close(fd);
		conn->flags |= CONN_DEAD;
		if (!(conn->flags & CONN_CONNECTING))
			sendto_all_dcc("Client %s disconnected", conn->name);
                delete_connection(fd);
		return 1;
        }

        va_end (ap);
	return 0;
}

int vsendto_dcc(int fd, char *fmt, va_list ap)
{
        char buf[BUFSIZE];
        struct Connection *conn;

        vsnprintf (buf, BUFSIZE, fmt, ap);

        debug("DCC SENT: %s", buf);
#ifdef IO_LOG
        iolog("DS: %s", buf);
#endif
        strlcat(buf, "\n", BUFSIZE);

        /* netinfo.outb += strlen(buf); */

        if ((write(fd, buf, strlen(buf))) == -1)
        {
                conn = find_connection(fd);
                log("Write error. Client %s FD: %d", conn->name, fd);
                close(fd);
                conn->flags |= CONN_DEAD;
                if (!(conn->flags & CONN_CONNECTING))
                        sendto_all_dcc("Client %s disconnected", conn->name);
                delete_connection(fd);
                return 1;
        }

        return 0;
}

static int sendto_dcc_nr(int fd, char *fmt,...)
{
        va_list ap;
        char buf[BUFSIZE];
        struct Connection *conn;

        va_start (ap, fmt);
        vsnprintf (buf, BUFSIZE, fmt, ap);

        debug("DCC SENT: %s", buf);
#ifdef IO_LOG
        iolog("DS: %s", buf);
#endif

        /* netinfo.outb += strlen(buf); */

        if ((write(fd, buf, strlen(buf))) == -1)
        {
                conn = find_connection(fd);
                log("Write error. Client %s FD: %d", conn->name, fd);
                close(fd);
                conn->flags |= CONN_DEAD;
                if (!(conn->flags & CONN_CONNECTING))
                        sendto_all_dcc("Client %s disconnected", conn->name);
                delete_connection(fd);
                return 1;
        }

        va_end (ap);
        return 0;
}

void sendto_all_dcc(char *fmt,...)
{
        va_list ap;
	dlink_node *dl;
	struct Connection *conn;

        va_start (ap, fmt);

	DLINK_FOREACH(dl, connection_list.head)
	{
		conn = (struct Connection *) dl->data;
		if (conn->flags & CONN_CONNECTING)
			continue;
		if (conn->flags & (CONN_DCCIN|CONN_DCCOUT))
		{
			if (conn->flags & CONN_DEAD)
				continue;
			vsendto_dcc(conn->fd, fmt, ap);
		}
	}

        va_end (ap);
}

void sendto_dcc_flag(int flag, char *fmt,...)
{
        va_list ap;
        dlink_node *dl;
        struct Connection *conn;

        va_start (ap, fmt);

        DLINK_FOREACH(dl, connection_list.head)
        {
                conn = (struct Connection *) dl->data;
		/* Check for the flag first, save us some time later */
		if (!(conn->flags & flag))
			continue;
                if (conn->flags & CONN_CONNECTING)
                        continue;
                if (conn->flags & (CONN_DCCIN|CONN_DCCOUT))
                {
                        if (conn->flags & CONN_DEAD)
                                continue;
                       	vsendto_dcc(conn->fd, fmt, ap);
                }
	}

	va_end(ap);
}

void sendto_partyline(struct Connection *sender, char *fmt,...)
{
        va_list ap;
        dlink_node *dl;
        struct Connection *conn;

        va_start (ap, fmt);

        DLINK_FOREACH(dl, connection_list.head)
        {
                conn = (struct Connection *) dl->data;
                if (conn->flags & CONN_CONNECTING)
                        continue;
                if (conn->flags & (CONN_DCCIN|CONN_DCCOUT))
                {
                        if (conn->flags & CONN_DEAD)
                                continue;
			if ((conn->flags & CONN_NOECHO) && (conn == sender))
				continue;
			if (conn->flags & CF_CHAT)
                        	vsendto_dcc(conn->fd, fmt, ap);
                }
        }

        va_end (ap);
}

/* Read a help file and send it to the recipient via DCC */
void send_dcc_help_file(int fd, char *filename, char *query)
{
        FILE *fp = fopen(filename, "r");
        char buffer[BUFSIZE];

        if (!fp)
        {
                sendto_dcc(fd, "No help available for: %s", query);
                log("No help available for %s, filename %s", query, filename);
                return;
        }

        while (fgets(buffer, BUFSIZE, fp))
        {
                if (buffer[0])
                        sendto_dcc_nr(fd, "%s", buffer);
                else
                        sendto_dcc(fd, " ");
        }

        fclose(fp);
}
