/* Stats - IRC Statistical Services for Hybrid, csircd, and IRCnet 2.10
** services.c - Functions to track IRCnet services
**
** Copyright W. Campbell and others.  See README for more details
** Some code Copyright: Jonathan George, Kai Seidler, ircd-hybrid Team,
**                      IRCnet IRCD developers.
**
** $Id: services.c,v 1.6 2002/10/04 15:01:23 wcampbel Exp $
*/

#include "stats.h"
#include "struct.h"
#include "proto.h"

#ifdef IRCD_2_10
struct Service *findservice(char *nick, char *server)
{
	struct Service *s = NULL;
	dlink_node *dl;

	debug("findservice(%s, %s)", nick, server);

	for (dl = servicehead.head; dl; dl = dl->next)
	{
		s = (struct Service *) dl->data;
		/* XXX - Do I need to match() the server? */
		if (!(strcasecmp(nick, s->nick)) &&
		    !(strcasecmp(server, s->server)))
			return s;
	}

	return NULL;
}

void AddService(char *nick, char *server, char *dist, char *gecos)
{
	struct Service *s = smalloc(sizeof(struct Service));
	dlink_node *dl = smalloc(sizeof(dlink_node));
	struct Server *sv = findserver(server);
	time_t current = time(NULL);

	strlcpy(s->nick, nick, MAXNICK + 1);
	strlcpy(s->server, server, MAXHOST);
	strlcpy(s->dist, dist, MAXHOST);
	strlcpy(s->gecos, gecos, LINE);
	dlink_add(s, dl, &servicehead);
	netinfo.services++;
        if (netinfo.services > netinfo.maxservices)
        {
                netinfo.maxservices = netinfo.services;
                netinfo.t_maxservices = current;
        }
	if (netinfo.services > daily.services)
	{
		daily.services = netinfo.services;
		daily.t_services = current;
	}
	if (sv)
	{
		sv->services++;
		if (sv->services > sv->stats->maxservices)
		{
			sv->stats->maxservices = sv->services;
			sv->stats->t_maxservices = current;
		}
	}
}

void DelService(char *nick, char *server)
{
	struct Service *s = NULL;
	struct Server *sv = findserver(server);
	dlink_node *dl;

	for (dl = servicehead.head; dl; dl = dl->next)
	{
		s = (struct Service *) dl->data;
		if (!(strcasecmp(nick, s->nick)) &&
		    !(strcasecmp(server, s->server)))
		{
			dlink_delete(dl, &servicehead);
			free(s);
			free(dl);
			netinfo.services--;
                        if (sv)
                                sv->services--;
                        return;
		}
	}

	log("Unable to delete service %s@%s", nick, server);
}

void DelServicesByServer(char *server)
{
	struct Service *s = NULL;
	dlink_node *dl = servicehead.head;
	dlink_node *tdl = NULL;
	/* struct Server *sv = findserver(server); */

	while (dl != NULL)
	{
		s = (struct Service *) dl->data;
		if (!(strcasecmp(server, s->server)))
		{
			tdl = dl->next;
			dlink_delete(dl, &servicehead);
			free(dl);
			free(s);
			netinfo.services--;
                        /* We're getting rid of this server record...
			** not necessary to decrease its services count
                        */
			dl = tdl;
		}
		else
		{
			dl = dl->next;
		}
	}
}
			
#endif

