/* Stats - IRC Statistical Services for Hybrid, csircd, and IRCnet 2.10
** split-data.c - Split service's backend functions
**
** Copyright W. Campbell and others.  See README for more details
** Some code Copyright: Jonathan George, Kai Seidler, ircd-hybrid Team,
**                      IRCnet IRCD developers.
**
** $Id: split-data.c,v 1.35 2002/09/29 03:03:07 wcampbel Exp $
*/

#include "config.h"
#include "stats.h"
#include "struct.h"
#include "proto.h"

#ifdef SPLITSERV

void SpInitTables()
{
	server_table.head = NULL;
	server_table.tail = NULL;
        map_head = SpNewServer(settings.name, NULL);
	split_table.head = NULL;
	split_table.tail = NULL;
}

struct SpServer* SpNewServer(char* name, char* uplink)
{
	struct SpServer* up = SpFindServer(uplink);
	struct SpServer* s = smalloc(sizeof(struct SpServer));
	dlink_node *dl;
	dlink_node *dupl;

	strlcpy(s->name, name, MAXHOST);
	/* No need to zero out the leaf list, smalloc() memsets */
	s->uplink = up;
	if (up)
	{
		up->numleaf++;
		dupl = smalloc(sizeof(dlink_node));
		dlink_add_tail(s, dupl, &up->leaflist);
	}
	else
	{
	    if (uplink != NULL) /* me... uplink will be NULL */
		log("SPLIT WARNING: Unable to find uplink: %s for server "
		    "name: %s", uplink, name);
	}
	dl = smalloc(sizeof(dlink_node));
	dlink_add_tail(s, dl, &server_table);
	return s;
}

void SpDelServer(char* name)
{
	struct SpServer* s = NULL;
	dlink_node *dl = server_table.head;
	dlink_node *dupl;
	struct SpServer *sup;

	if (name != NULL)
	{
		while (dl != NULL)
		{
			s = (struct SpServer *) dl->data;
			if (!strcasecmp(s->name, name))
			{
				dlink_delete(dl, &server_table);
				free(dl);
				break;
			}
			/* Safe for glibc */
			dl = dl->next;
		}
		if (s)
		{
		    if (s->uplink)
		    {
			s->uplink->numleaf--;

			dupl = s->uplink->leaflist.head;
			while (dupl != NULL)
			{
				sup = (struct SpServer *) dupl->data;
				if (!strcasecmp(sup->name, name))
				{
					dlink_delete(dupl,
						&s->uplink->leaflist);
					free(dupl);
					break;
				}
				/* safe for glibc */
				dupl = dupl->next;
			}
                    }
		    else
		    {
                        log("SPLIT: No uplink for server: %s", s->name);
		    }

		    free(s);
		}
		else
		{
		    log("SPLIT: No server table entry for server name: %s",
			name);
		}
	}
}

struct SpServer* SpFindServer(char* name)
{
	dlink_node *dl = server_table.head;
	struct SpServer *s;
	
	if (name)
	{
		while (dl != NULL)
		{
			s = (struct SpServer *) dl->data;
			if (!strcasecmp(s->name, name))
			{
				return s;
			}
			dl = dl->next;
		}
	}
	debug("SPLIT: Unable to FindServer for name: %s", name);
	return NULL;
}

void SpNewSplit(char * name)
{
	struct SpServer* s;
	struct SpServer *t = SpFindSplit(name);
	dlink_node *dl;
	if (t)
	{
		log("SPLIT: Split entry for server name: %s already exists",
			name);
		return;
	}
	s = smalloc(sizeof(struct SpServer));
	dl = smalloc(sizeof(dlink_node));
	strlcpy(s->name, name, MAXHOST);
	s->ts = time(NULL);
	/* None of the other variables are used here */
	dlink_add_tail(s, dl, &split_table);
}

void SpDelSplit(char * name)
{
	struct SpServer* s = NULL;
	dlink_node *dl = split_table.head;
	while (dl)
	{
		s = (struct SpServer *) dl->data;
		if (!strcasecmp(s->name, name))
		{
			dlink_delete(dl, &split_table);
			free(s);
			free(dl);
			return;
		}
		/* This is safe, won't cause Linux cores */
		dl = dl->next;
	}
	/* New servers will get logged if logging is done here, too much
	** useless clutter in the log file
	*/
}

struct SpServer* SpFindSplit(char* name)
{
	dlink_node *dl = NULL;
	struct SpServer *spl = NULL;
        if (name)
	{
		for (dl = split_table.head; dl; dl = dl->next)
		{
			spl = (struct SpServer *) dl->data;
			if (!strcasecmp(spl->name, name))
			{
				return spl;
			}
		}
        }
	debug("SPLIT: Unable to FindSplit server name: %s", name);
        return NULL;
}

#endif
